@@ -6,6 +6,8 @@ CDirectoryListing::CDirectoryListing()
 	m_hasUnsureEntries = 0;
 	m_failed = false;
 	m_hasDirs = false;
+	m_has_perms = false;
+	m_has_usergroup = false;
 }
 
 CDirectoryListing::CDirectoryListing(const CDirectoryListing& listing)
@@ -21,6 +23,10 @@ CDirectoryListing::CDirectoryListing(const CDirectoryListing& listing)
 	m_firstListTime = listing.m_firstListTime;
 
 	m_hasDirs = listing.m_hasDirs;
+
+	m_has_perms = listing.m_has_perms;
+	m_has_usergroup = listing.m_has_usergroup;
+
 }
 
 CDirectoryListing& CDirectoryListing::operator=(const CDirectoryListing &a)
@@ -44,18 +50,21 @@ CDirectoryListing& CDirectoryListing::operator=(const CDirectoryListing &a)
 	m_searchmap_case = a.m_searchmap_case;
 	m_searchmap_nocase = a.m_searchmap_nocase;
 
+	m_has_perms = a.m_has_perms;
+	m_has_usergroup = a.m_has_usergroup;
+
 	return *this;
 }
 
 wxString CDirentry::dump() const
 {
-	wxString str = wxString::Format(_T("name=%s\nsize=%s\npermissions=%s\nownerGroup=%s\ndir=%d\nlink=%d\ntarget=%s\nhasDate=%d\nhasTime=%d\nunsure=%d\n"),
+	wxString str = wxString::Format(_T("name=%s\nsize=%s\npermissions=%s\nownerGroup=%s\ndir=%d\nlink=%d\ntarget=%s\nhasTimestamp=%d\nunsure=%d\n"),
 				name.c_str(), size.ToString().c_str(), permissions.c_str(), ownerGroup.c_str(), dir, link,
-				target.c_str(), hasDate, hasTime, unsure);
+				target.c_str(), hasTimestamp, unsure);
 
-	if (hasDate)
+	if (hasTimestamp != timestamp_none)
 		str += _T("date=") + time.FormatISODate() + _T("\n");
-	if (hasTime)
+	if (hasTimestamp >= timestamp_time)
 		str += _T("time=") + time.FormatISOTime() + _T("\n");
 	str += wxString::Format(_T("unsure=%d\n"), unsure);
 	return str;
@@ -84,13 +93,10 @@ bool CDirentry::operator==(const CDirentry &op) const
 	if (target != op.target)
 		return false;
 
-	if (hasDate != op.hasDate)
+	if (hasTimestamp != op.hasTimestamp)
 		return false;
 
-	if (hasTime != op.hasTime)
-		return false;
-
-	if (hasDate)
+	if (hasTimestamp != timestamp_none)
 	{
 		if (time != op.time)
 			return false;
@@ -149,11 +155,17 @@ void CDirectoryListing::Assign(const std::list<CDirentry> &entries)
 	own_entries.reserve(m_entryCount);
 	
 	m_hasDirs = false;
+	m_has_perms = false;
+	m_has_usergroup = false;
 	
 	for (std::list<CDirentry>::const_iterator iter = entries.begin(); iter != entries.end(); iter++)
 	{
 		if (iter->dir)
 			m_hasDirs = true;
+		if (!iter->permissions.empty())
+			m_has_usergroup = true;
+		if (!iter->ownerGroup.empty())
+			m_has_perms = true;
 		own_entries.push_back(CRefcountObject<CDirentry>(*iter));
 	}
 
