@@ -29,8 +29,7 @@ typedef void (wxEvtHandler::*fzEngineEventFunction)(wxFzEngineEvent&);
     ),
 
 std::list<CFileZillaEnginePrivate*> CFileZillaEnginePrivate::m_engineList;
-int CFileZillaEnginePrivate::m_activeStatusSend = 0;
-int CFileZillaEnginePrivate::m_activeStatusRecv = 0;
+int CFileZillaEnginePrivate::m_activeStatus[2] = {0, 0};
 std::list<CFileZillaEnginePrivate::t_failedLogins> CFileZillaEnginePrivate::m_failedLogins;
 
 DEFINE_EVENT_TYPE(fzEVT_ENGINE_NOTIFICATION);
@@ -186,24 +185,25 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 
 		if (m_pCurrentCommand->GetId() == cmd_connect)
 		{
-			if (!(nErrorCode & ~(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED | FZ_REPLY_TIMEOUT)) && 
-				nErrorCode & (FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED) &&
-				m_retryCount < m_pOptions->GetOptionVal(OPTION_RECONNECTCOUNT))
+			if (!(nErrorCode & ~(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED | FZ_REPLY_TIMEOUT | FZ_REPLY_CRITICALERROR | FZ_REPLY_PASSWORDFAILED)) && 
+				nErrorCode & (FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED))				
 			{
-				m_retryCount++;
-
 				const CConnectCommand *pConnectCommand = (CConnectCommand *)m_pCurrentCommand;
 
-				RegisterFailedLoginAttempt(pConnectCommand->GetServer());
+				RegisterFailedLoginAttempt(pConnectCommand->GetServer(), (nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR);
 
-				if (pConnectCommand->RetryConnecting())
+				if ((nErrorCode & FZ_REPLY_CRITICALERROR) != FZ_REPLY_CRITICALERROR)
 				{
-					unsigned int delay = GetRemainingReconnectDelay(pConnectCommand->GetServer());
-					if (!delay)
-						delay = 1;
-					m_pLogging->LogMessage(Status, _("Waiting to retry..."));
-					m_retryTimer.Start(delay, true);
-					return FZ_REPLY_WOULDBLOCK;
+					m_retryCount++;
+					if (m_retryCount < m_pOptions->GetOptionVal(OPTION_RECONNECTCOUNT) && pConnectCommand->RetryConnecting())
+					{
+						unsigned int delay = GetRemainingReconnectDelay(pConnectCommand->GetServer());
+						if (!delay)
+							delay = 1;
+						m_pLogging->LogMessage(Status, _("Waiting to retry..."));
+						m_retryTimer.Start(delay, true);
+						return FZ_REPLY_WOULDBLOCK;
+					}
 				}
 			}
 		}
@@ -235,27 +235,14 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 	return nErrorCode;
 }
 
-void CFileZillaEnginePrivate::SetActive(bool recv)
+void CFileZillaEnginePrivate::SetActive(int direction)
 {
 	if (m_pControlSocket)
 		m_pControlSocket->SetAlive();
 
-	if (recv)
-	{
-		if (!m_activeStatusRecv)
-		{
-			m_activeStatusRecv = 2;
-			AddNotification(new CActiveNotification(true));
-		}
-	}
-	else
-	{
-		if (!m_activeStatusSend)
-		{
-			m_activeStatusSend = 2;
-			AddNotification(new CActiveNotification(false));
-		}
-	}
+	if (!m_activeStatus[direction])
+		AddNotification(new CActiveNotification(direction));
+	m_activeStatus[direction] = 2;
 }
 
 unsigned int CFileZillaEnginePrivate::GetNextAsyncRequestNumber()
@@ -274,6 +261,17 @@ int CFileZillaEnginePrivate::Connect(const CConnectCommand &command)
 		return FZ_REPLY_BUSY;
 
 	m_retryCount = 0;
+
+	if (m_pControlSocket)
+	{
+		// Need to delete before setting m_pCurrentCommand.
+		// The destructor can call CFileZillaEnginePrivate::ResetOperation
+		// which would delete m_pCurrentCommand
+		delete m_pControlSocket;
+		m_pControlSocket = 0;
+		m_nControlSocketError = 0;
+	}
+
 	m_pCurrentCommand = command.Clone();
 
 	if (command.GetServer().GetPort() != CServer::GetDefaultPort(command.GetServer().GetProtocol()))
@@ -355,6 +353,8 @@ int CFileZillaEnginePrivate::List(const CListCommand &command)
 		if (pServer)
 		{
 			CServerPath path(CPathCache::Lookup(*pServer, command.GetPath(), command.GetSubDir()));
+			if (path.IsEmpty() && command.GetSubDir().IsEmpty())
+				path = command.GetPath();
 			if (!path.IsEmpty())
 			{
 				CDirectoryListing *pListing = new CDirectoryListing;
@@ -553,14 +553,14 @@ void CFileZillaEnginePrivate::SendDirectoryListingNotification(const CServerPath
 	}
 }
 
-void CFileZillaEnginePrivate::RegisterFailedLoginAttempt(const CServer& server)
+void CFileZillaEnginePrivate::RegisterFailedLoginAttempt(const CServer& server, bool critical)
 {
 	std::list<t_failedLogins>::iterator iter = m_failedLogins.begin();
 	while (iter != m_failedLogins.end())
 	{
 		const wxTimeSpan span = wxDateTime::UNow() - iter->time;
 		if (span.GetSeconds() >= m_pOptions->GetOptionVal(OPTION_RECONNECTDELAY) ||
-			(iter->host == server.GetHost() && iter->port == server.GetPort()))
+			iter->server == server || (!critical && (iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort())))
 		{
 			std::list<t_failedLogins>::iterator prev = iter;
 			iter++;
@@ -571,8 +571,7 @@ void CFileZillaEnginePrivate::RegisterFailedLoginAttempt(const CServer& server)
 	}
 
 	t_failedLogins failure;
-	failure.host = server.GetHost();
-	failure.port = server.GetPort();
+	failure.server = server;
 	failure.time = wxDateTime::UNow();
 	m_failedLogins.push_back(failure);
 }
@@ -590,10 +589,10 @@ unsigned int CFileZillaEnginePrivate::GetRemainingReconnectDelay(const CServer&
 			iter++;
 			m_failedLogins.erase(prev);
 		}
-		else if (iter->host == server.GetHost() && iter->port == server.GetPort())
-		{
+		else if (!iter->critical && iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort())
+			return delay * 1000 - span.GetMilliseconds().GetLo();
+		else if (iter->server == server)
 			return delay * 1000 - span.GetMilliseconds().GetLo();
-		}
 		else
 			iter++;
 	}
@@ -615,23 +614,23 @@ void CFileZillaEnginePrivate::OnTimer(wxTimerEvent& event)
 	wxASSERT(!GetRemainingReconnectDelay(pConnectCommand->GetServer()));
 #endif
 
-	ContinueConnect();
-}
-
-int CFileZillaEnginePrivate::ContinueConnect()
-{
 	if (m_pControlSocket)
 	{
 		delete m_pControlSocket;
 		m_pControlSocket = 0;
 	}
 
+	ContinueConnect();
+}
+
+int CFileZillaEnginePrivate::ContinueConnect()
+{
 	const CConnectCommand *pConnectCommand = (CConnectCommand *)m_pCurrentCommand;
 	const CServer& server = pConnectCommand->GetServer();
 	unsigned int delay = GetRemainingReconnectDelay(server);
 	if (delay)
 	{
-		m_pLogging->LogMessage(Status, _("Delaying connection due to previously failed connection attempt..."));
+		m_pLogging->LogMessage(Status, wxPLURAL("Delaying connection for %d second due to previously failed connection attempt...", "Delaying connection for %d seconds due to previously failed connection attempt...", (delay + 999) / 1000), (delay + 999) / 1000);
 		m_retryTimer.Start(delay, true);
 		return FZ_REPLY_WOULDBLOCK;
 	}
@@ -647,6 +646,7 @@ int CFileZillaEnginePrivate::ContinueConnect()
 		m_pControlSocket = new CSftpControlSocket(this);
 		break;
 	case HTTP:
+	case HTTPS:
 		m_pControlSocket = new CHttpControlSocket(this);
 		break;
 	default:
