@@ -604,7 +604,7 @@ protected:
 
 		struct addrinfo *addressList = 0;
 		struct addrinfo hints = {0};
-		hints.ai_family = AF_UNSPEC;
+		hints.ai_family = m_pSocket->m_family;
 		hints.ai_socktype = SOCK_STREAM;
 
 #ifdef __WXMSW__
@@ -1069,6 +1069,7 @@ CSocket::CSocket(CSocketEventHandler* pEvtHandler)
 	m_flags = 0;
 
 	m_port = 0;
+	m_family = AF_UNSPEC;
 
 	m_buffer_sizes[0] = -1;
 	m_buffer_sizes[1] = -1;
@@ -1116,7 +1117,7 @@ void CSocket::DetachThread()
 	Cleanup(false);
 }
 
-int CSocket::Connect(wxString host, unsigned int port)
+int CSocket::Connect(wxString host, unsigned int port, enum address_family family /*=unsped*/)
 {
 	if (m_state != none)
 		return EISCONN;
@@ -1124,6 +1125,21 @@ int CSocket::Connect(wxString host, unsigned int port)
 	if (port < 1 || port > 65535)
 		return EINVAL;
 
+	switch (family)
+	{
+	case unspec:
+		m_family = AF_UNSPEC;
+		break;
+	case ipv4:
+		m_family = AF_INET;
+		break;
+	case ipv6:
+		m_family = AF_INET6;
+		break;
+	default:
+		return EINVAL;
+	}
+
 	if (m_pSocketThread && m_pSocketThread->m_started)
 	{
 		m_pSocketThread->m_sync.Lock();
@@ -1337,32 +1353,42 @@ wxString CSocket::GetErrorDescription(int error)
 
 int CSocket::Close()
 {
-	if (m_fd != -1)
+	int fd;
+	if (m_pSocketThread)
+	{
+		m_pSocketThread->m_sync.Lock();
+		fd = m_fd;
+		m_fd = -1;
+		delete [] m_pSocketThread->m_pHost;
+		m_pSocketThread->m_pHost = 0;
+		delete [] m_pSocketThread->m_pPort;
+		m_pSocketThread->m_pPort = 0;
+		if (!m_pSocketThread->m_threadwait)
+			m_pSocketThread->WakeupThread(true);
+	}
+	else
+	{
+		fd = m_fd;
+		m_fd = -1;
+	}
+
+	if (fd != -1)
 	{
-		int fd = m_fd;
-		if (m_pSocketThread)
-		{
-			m_pSocketThread->m_sync.Lock();
-			m_fd = -1;
-			if (!m_pSocketThread->m_threadwait)
-				m_pSocketThread->WakeupThread(true);
-		}
-		else
-			m_fd = -1;
-			
 #ifdef __WXMSW__
 		closesocket(fd);
 #else
 		close(fd);
 #endif
-		if (m_pSocketThread)
-			m_pSocketThread->m_sync.Unlock();
-
-		if (m_pEvtHandler)
-			CSocketEventDispatcher::Get().RemovePending(m_pEvtHandler);
 	}
+
 	m_state = none;
 
+	if (m_pSocketThread)
+		m_pSocketThread->m_sync.Unlock();
+
+	if (m_pEvtHandler)
+		CSocketEventDispatcher::Get().RemovePending(m_pEvtHandler);
+
 	return 0;
 }
 
@@ -1565,18 +1591,26 @@ wxString CSocket::GetPeerIP() const
 	return AddressToString((sockaddr *)&addr, addr_len, false);
 }
 
-int CSocket::GetAddressFamily() const
+enum CSocket::address_family CSocket::GetAddressFamily() const
 {
 	struct sockaddr_storage addr;
 	socklen_t addr_len = sizeof(addr);
 	int res = getsockname(m_fd, (sockaddr*)&addr, &addr_len);
 	if (res)
-		return AF_UNSPEC;
+		return unspec;
 
-	return ((sockaddr*)&addr)->sa_family;
+	switch (((sockaddr*)&addr)->sa_family)
+	{
+	case AF_INET:
+		return ipv4;
+	case AF_INET6:
+		return ipv6;
+	default:
+		return unspec;
+	}
 }
 
-int CSocket::Listen(int family, int port /*=0*/)
+int CSocket::Listen(enum address_family family, int port /*=0*/)
 {
 	if (m_state != none)
 		return EALREADY;
@@ -1584,6 +1618,21 @@ int CSocket::Listen(int family, int port /*=0*/)
 	if (port < 0 || port > 65535)
 		return EINVAL;
 
+	switch (family)
+	{
+	case unspec:
+		m_family = AF_UNSPEC;
+		break;
+	case ipv4:
+		m_family = AF_INET;
+		break;
+	case ipv6:
+		m_family = AF_INET6;
+		break;
+	default:
+		return EINVAL;
+	}
+
 #ifdef __WXMSW__
 	if (!getaddrinfo)
 	{
@@ -1611,7 +1660,7 @@ int CSocket::Listen(int family, int port /*=0*/)
 #endif
 	{
 		struct addrinfo hints = {0};
-		hints.ai_family = family;
+		hints.ai_family = m_family;
 		hints.ai_socktype = SOCK_STREAM;
 		hints.ai_flags = AI_PASSIVE;
 #ifdef AI_NUMERICSERV
