@@ -1,9 +1,3 @@
-#include <wx/defs.h>
-#ifdef __WXMSW__
-// For AF_INET6
-#include <winsock2.h>
-#include <ws2tcpip.h>
-#endif
 #include "FileZilla.h"
 #include "transfersocket.h"
 #include "ftpcontrolsocket.h"
@@ -12,9 +6,6 @@
 #include "iothread.h"
 #include "tlssocket.h"
 #include <errno.h>
-#ifndef __WXMSW__
-#include <sys/socket.h>
-#endif
 #include "proxy.h"
 #include "servercapabilities.h"
 
@@ -113,7 +104,7 @@ wxString CTransferSocket::SetupActiveTransfer(const wxString& ip)
 	}
 
 	wxString portArguments;
-	if (m_pSocketServer->GetAddressFamily() == AF_INET6)
+	if (m_pSocketServer->GetAddressFamily() == CSocket::ipv6)
 	{
 		portArguments = wxString::Format(_T("|2|%s|%d|"), ip.c_str(), port);
 	}
@@ -297,7 +288,7 @@ void CTransferSocket::OnReceive()
 			if (numread > 0)
 			{
 				m_pDirectoryListingParser->AddData(pBuffer, numread);
-				m_pEngine->SetActive(true);
+				m_pEngine->SetActive(CFileZillaEngine::recv);
 				if (!m_madeProgress)
 				{
 					m_madeProgress = 2;
@@ -336,7 +327,7 @@ void CTransferSocket::OnReceive()
 
 			if (numread > 0)
 			{
-				m_pEngine->SetActive(true);
+				m_pEngine->SetActive(CFileZillaEngine::recv);
 				if (!m_madeProgress)
 				{
 					m_madeProgress = 2;
@@ -433,7 +424,7 @@ void CTransferSocket::OnSend()
 		if (written <= 0)
 			break;
 
-		m_pEngine->SetActive(false);
+		m_pEngine->SetActive(CFileZillaEngine::send);
 		if (m_madeProgress == 1)
 		{
 			m_pControlSocket->LogMessage(::Debug_Debug, _T("Made progress in CTransferSocket::OnSend()"));
