@@ -248,11 +248,12 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_COMMAND(-1, fzEVT_VOLUMESENUMERATED, CLocalListView::OnVolumesEnumerated)
 	EVT_COMMAND(-1, fzEVT_VOLUMEENUMERATED, CLocalListView::OnVolumesEnumerated)
 #endif
+	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CLocalListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
 CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQueue)
 	: CFileListCtrl<CLocalFileData>(pParent, pState, pQueue),
-	CSystemImageList(16), CStateEventHandler(pState)
+	CStateEventHandler(pState)
 {
 	m_pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
 	m_pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
@@ -262,7 +263,7 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQ
 
 	const unsigned long widths[4] = { 120, 80, 100, 120 };
 
-	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0]);
+	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0], true);
 	AddColumn(_("Filesize"), wxLIST_FORMAT_RIGHT, widths[1]);
 	AddColumn(_("Filetype"), wxLIST_FORMAT_LEFT, widths[2]);
 	AddColumn(_("Last modified"), wxLIST_FORMAT_LEFT, widths[3]);
@@ -274,9 +275,9 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQ
 
 #ifdef __WXMSW__
 	m_pVolumeEnumeratorThread = 0;
+#endif
 
 	InitHeaderImageList();
-#endif
 
 	SetDropTarget(new CLocalListViewDropTarget(this));
 
@@ -2489,3 +2490,8 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 }
 
 #endif
+
+void CLocalListView::OnMenuRefresh(wxCommandEvent& event)
+{
+	m_pState->RefreshLocal();
+}
