@@ -198,9 +198,9 @@ protected:
 	wxDataObjectComposite* m_pDataObject;
 };
 
-IMPLEMENT_CLASS(CLocalTreeView, wxTreeCtrl)
+IMPLEMENT_CLASS(CLocalTreeView, wxTreeCtrlEx)
 
-BEGIN_EVENT_TABLE(CLocalTreeView, wxTreeCtrl)
+BEGIN_EVENT_TABLE(CLocalTreeView, wxTreeCtrlEx)
 EVT_TREE_ITEM_EXPANDING(wxID_ANY, CLocalTreeView::OnItemExpanding)
 #ifdef __WXMSW__
 EVT_TREE_SEL_CHANGING(wxID_ANY, CLocalTreeView::OnSelectionChanging)
@@ -215,17 +215,18 @@ EVT_COMMAND(-1, fzEVT_VOLUMEENUMERATED, CLocalTreeView::OnVolumesEnumerated)
 #endif //__WXMSW__
 EVT_TREE_ITEM_MENU(wxID_ANY, CLocalTreeView::OnContextMenu)
 EVT_MENU(XRCID("ID_UPLOAD"), CLocalTreeView::OnMenuUpload)
-EVT_MENU(XRCID("ID_ADDTOQUEUE"), CLocalTreeView::OnMenuUpload)
+EVT_MENU(XRCID("ID_ADDTO	QUEUE"), CLocalTreeView::OnMenuUpload)
 EVT_MENU(XRCID("ID_DELETE"), CLocalTreeView::OnMenuDelete)
 EVT_MENU(XRCID("ID_RENAME"), CLocalTreeView::OnMenuRename)
 EVT_MENU(XRCID("ID_MKDIR"), CLocalTreeView::OnMenuMkdir)
 EVT_TREE_BEGIN_LABEL_EDIT(wxID_ANY, CLocalTreeView::OnBeginLabelEdit)
 EVT_TREE_END_LABEL_EDIT(wxID_ANY, CLocalTreeView::OnEndLabelEdit)
 EVT_CHAR(CLocalTreeView::OnChar)
+EVT_MENU(XRCID("ID_OPEN"), CLocalTreeView::OnMenuOpen)
 END_EVENT_TABLE()
 
 CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState, CQueueView *pQueueView)
-	: wxTreeCtrl(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER),
+	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER),
 	CSystemImageList(16),
 	CStateEventHandler(pState),
 	m_pQueueView(pQueueView)
@@ -237,8 +238,6 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState,
 	pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
 	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
 
-	m_setSelection = false;
-
 	SetImageList(GetSystemImageList());
 
 #ifdef __WXMSW__
@@ -246,7 +245,12 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState,
 
 	CreateRoot();
 #else
-	AddRoot(_T("/"), GetIconIndex(dir), GetIconIndex(opened_dir));
+	wxTreeItemId root = AddRoot(_T("/"));
+	SetItemImage(root, GetIconIndex(dir), wxTreeItemIcon_Normal);
+	SetItemImage(root, GetIconIndex(opened_dir), wxTreeItemIcon_Selected);
+	SetItemImage(root, GetIconIndex(dir), wxTreeItemIcon_Expanded);
+	SetItemImage(root, GetIconIndex(opened_dir), wxTreeItemIcon_SelectedExpanded);
+
 	SetDir(_T("/"));
 #endif
 
@@ -272,9 +276,7 @@ void CLocalTreeView::SetDir(wxString localDir)
 	{
 		// TODO: UNC path, don't display it yet
 		m_currentDir = _T("");
-		m_setSelection = true;
-		SelectItem(wxTreeItemId());
-		m_setSelection = false;
+		SafeSelectItem(wxTreeItemId());
 		return;
 	}
 	m_currentDir = localDir;
@@ -282,9 +284,7 @@ void CLocalTreeView::SetDir(wxString localDir)
 #ifdef __WXMSW__
 	if (localDir == _T("\\"))
 	{
-		m_setSelection = true;
-		SelectItem(m_drives);
-		m_setSelection = false;
+		SafeSelectItem(m_drives);
 		return;
 	}
 #endif
@@ -293,9 +293,7 @@ void CLocalTreeView::SetDir(wxString localDir)
 	wxTreeItemId parent = GetNearestParent(subDirs);
 	if (!parent)
 	{
-		m_setSelection = true;
-		SelectItem(wxTreeItemId());
-		m_setSelection = false;
+		SafeSelectItem(wxTreeItemId());
 		return;
 	}
 
@@ -308,22 +306,14 @@ void CLocalTreeView::SetDir(wxString localDir)
 		//if (child && GetItemText(child) == _T(""))
 		//	DisplayDir(parent, localDir);
 
-		m_setSelection = true;
-		SelectItem(parent);
-		m_setSelection = false;
-		if (parent != GetRootItem())
-			Expand(GetItemParent(parent));
+		SafeSelectItem(parent);
 		return;
 	}
 	wxTreeItemId item = MakeSubdirs(parent, localDir.Left(localDir.Length() - subDirs.Length()), subDirs);
 	if (!item)
 		return;
 
-	m_setSelection = true;
-	SelectItem(item);
-	m_setSelection = false;
-	if (item != GetRootItem())
-		Expand(GetItemParent(item));
+	SafeSelectItem(item);
 }
 
 wxTreeItemId CLocalTreeView::GetNearestParent(wxString& localDir)
@@ -1611,6 +1601,10 @@ void CLocalTreeView::OnSelectionChanging(wxTreeEvent& event)
 {
 	// On-demand open icon for selected items
 	wxTreeItemId item = event.GetItem();
+
+	if (!item)
+		return;
+
 	if (GetItemImage(item, wxTreeItemIcon_Selected) == -1)
 	{
 		int icon = GetIconIndex(opened_dir, GetDirFromItem(item));
@@ -1620,3 +1614,19 @@ void CLocalTreeView::OnSelectionChanging(wxTreeEvent& event)
 }
 
 #endif
+
+void CLocalTreeView::OnMenuOpen(wxCommandEvent& event)
+{
+	if (!m_contextMenuItem.IsOk())
+		return;
+
+	wxString path = GetDirFromItem(m_contextMenuItem);
+	if (path == _T(""))
+		return;
+
+	wxString url = CState::GetAsURL(path);
+	if (url == _T(""))
+		return;
+
+	wxLaunchDefaultBrowser(url);
+}
