@@ -7,29 +7,25 @@
 
 class CStatusView;
 class CQueueView;
-class CLocalTreeView;
-class CLocalListView;
-class CRemoteTreeView;
-class CRemoteListView;
 class CState;
 class CAsyncRequestQueue;
 class CLed;
 class CThemeProvider;
-class CView;
 class CQuickconnectBar;
 #if FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
 class CUpdateWizard;
 #endif //FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
 class CSiteManagerItemData_Site;
 class CQueue;
-class CViewHeader;
 class CWindowStateManager;
 class CStatusBar;
 class CMainFrameStateEventHandler;
 class CSplitterWindowEx;
+class CContextControl;
 
 class CMainFrame : public wxFrame
 {
+	friend class CMainFrameStateEventHandler;
 public:
 	CMainFrame();
 	virtual ~CMainFrame();
@@ -37,10 +33,7 @@ public:
 	void UpdateActivityLed(int direction);
 
 	void AddToRequestQueue(CFileZillaEngine* pEngine, CAsyncRequestNotification* pNotification);
-	CState* GetState() { return m_pState; }
 	CStatusView* GetStatusView() { return m_pStatusView; }
-	CLocalListView* GetLocalListView() { return m_pLocalListView; }
-	CRemoteListView* GetRemoteListView() { return m_pRemoteListView; }
 	CQueueView* GetQueue() { return m_pQueueView; }
 	CQuickconnectBar* GetQuickconnectBar() { return m_pQuickconnectBar; }
 
@@ -63,7 +56,11 @@ public:
 	void ProcessCommandLine();
 
 	void ClearBookmarks();
+
+	void PostInitialize();
 	
+	bool Connect(const CServer& server, const CServerPath& path = CServerPath());
+
 protected:
 	bool CreateMenus();
 	bool CreateQuickconnectBar();
@@ -76,13 +73,12 @@ protected:
 
 	void FocusNextEnabled(std::list<wxWindow*>& windowOrder, std::list<wxWindow*>::iterator iter, bool skipFirst, bool forward);
 
+	void SetBookmarksFromPath(const wxString& path);
 	void UpdateBookmarkMenu();
 
 	std::list<int> m_bookmark_menu_ids;
 	std::map<int, wxString> m_bookmark_menu_id_map_global;
 	std::map<int, wxString> m_bookmark_menu_id_map_site;
-	wxString m_last_bookmark_path;
-	std::list<wxString> m_bookmarks;
 
 	CStatusBar* m_pStatusBar;
 	wxMenuBar* m_pMenuBar;
@@ -90,24 +86,13 @@ protected:
 	CQuickconnectBar* m_pQuickconnectBar;
 
 	CSplitterWindowEx* m_pTopSplitter; // If log position is 0, splits message log from rest of panes
-	CSplitterWindowEx* m_pBottomSplitter; // Top contains view splitter, buttom queue (or queuelog splitter if in position 1)
-	CSplitterWindowEx* m_pViewSplitter; // Contains local and remote splitters
-	CSplitterWindowEx* m_pLocalSplitter;
-	CSplitterWindowEx* m_pRemoteSplitter;
+	CSplitterWindowEx* m_pBottomSplitter; // Top contains view splitter, bottom queue (or queuelog splitter if in position 1)
 	CSplitterWindowEx* m_pQueueLogSplitter;
 
+	CContextControl* m_pContextControl;
+
 	CStatusView* m_pStatusView;
 	CQueueView* m_pQueueView;
-	CView* m_pLocalTreeViewPanel;
-	CView* m_pLocalListViewPanel;
-	CLocalTreeView* m_pLocalTreeView;
-	CLocalListView* m_pLocalListView;
-	CView* m_pRemoteTreeViewPanel;
-	CView* m_pRemoteListViewPanel;
-	CRemoteTreeView* m_pRemoteTreeView;
-	CRemoteListView* m_pRemoteListView;
-	CViewHeader* m_pLocalViewHeader;
-	CViewHeader* m_pRemoteViewHeader;
 	CLed* m_pActivityLed[2];
 	wxTimer m_transferStatusTimer;
 	CThemeProvider* m_pThemeProvider;
@@ -167,8 +152,13 @@ protected:
 #ifndef __WXMAC__
 	void OnIconize(wxIconizeEvent& event);
 	void OnTaskBarClick(wxTaskBarIconEvent& event);
+#endif
+#ifdef __WXGTK__
+	void OnTaskBarClick_Delayed(wxCommandEvent& event);
 #endif
 	void OnSearch(wxCommandEvent& event);
+	void OnMenuNewTab(wxCommandEvent& event);
+	void OnMenuCloseTab(wxCommandEvent& event);
 
 	bool m_bInitDone;
 	bool m_bQuit;
@@ -176,7 +166,6 @@ protected:
 	wxTimer m_closeEventTimer;
 
 	CAsyncRequestQueue* m_pAsyncRequestQueue;
-	CState* m_pState;
 	CMainFrameStateEventHandler* m_pStateEventHandler;
 
 	CWindowStateManager* m_pWindowStateManager;
@@ -186,6 +175,14 @@ protected:
 #ifndef __WXMAC__
 	wxTaskBarIcon* m_taskBarIcon;
 #endif
+#ifdef __WXGTK__
+	// There is a bug in KDE, causing the window to toggle iconized state
+	// several times a second after uniconizing it from taskbar icon.
+	// Set m_taskbar_is_uniconizing in OnTaskBarClick and unset the
+	// next time the pending event processing runs and calls OnTaskBarClick_Delayed.
+	// While set, ignore iconize events.
+	bool m_taskbar_is_uniconizing;
+#endif
 };
 
 #endif
