@@ -144,7 +144,11 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Search sort order", string, _T(""), false },
 	{ "Edit track local", number, _T("1"), false },
 	{ "Prevent idle sleep", number, _T("1"), false },
-	{ "Filteredit window size", string, _T(""), false }
+	{ "Filteredit window size", string, _T(""), false },
+	{ "Enable invalid char filter", number, _T("1"), false },
+	{ "Invalid char replace", string, _T("_"), false },
+	{ "Already connected choice", number, _T("0"), false },
+	{ "Edit status dialog size", string, _T(""), false }
 };
 
 struct t_default_option
@@ -312,8 +316,8 @@ void COptions::CreateSettingsXmlElement()
 	if (element->FirstChildElement("Settings"))
 		return;
 
-	TiXmlElement settings("Settings");
-	element->InsertEndChild(settings);
+	TiXmlElement *settings = new TiXmlElement("Settings");
+	element->LinkEndChild(settings);
 
 	for (int i = 0; i < OPTIONS_NUM; i++)
 	{
@@ -346,7 +350,7 @@ void COptions::SetXmlValue(unsigned int nID, wxString value)
 	TiXmlElement *settings = m_pXmlFile->GetElement()->FirstChildElement("Settings");
 	if (!settings)
 	{
-		TiXmlNode *node = m_pXmlFile->GetElement()->InsertEndChild(TiXmlElement("Settings"));
+		TiXmlNode *node = m_pXmlFile->GetElement()->LinkEndChild(new TiXmlElement("Settings"));
 		if (!node)
 		{
 			delete [] utf8;
@@ -377,18 +381,18 @@ void COptions::SetXmlValue(unsigned int nID, wxString value)
 			setting->RemoveAttribute("type");
 			setting->Clear();
 			setting->SetAttribute("type", (options[nID].type == string) ? "string" : "number");
-			setting->InsertEndChild(TiXmlText(utf8));
+			setting->LinkEndChild(new TiXmlText(utf8));
 
 			delete [] utf8;
 			return;
 		}
 	}
 	wxASSERT(options[nID].name[0]);
-	TiXmlElement setting("Setting");
-	setting.SetAttribute("name", options[nID].name);
-	setting.SetAttribute("type", (options[nID].type == string) ? "string" : "number");
-	setting.InsertEndChild(TiXmlText(utf8));
-	settings->InsertEndChild(setting);
+	TiXmlElement *setting = new TiXmlElement("Setting");
+	setting->SetAttribute("name", options[nID].name);
+	setting->SetAttribute("type", (options[nID].type == string) ? "string" : "number");
+	setting->LinkEndChild(new TiXmlText(utf8));
+	settings->LinkEndChild(setting);
 
 	delete [] utf8;
 }
@@ -403,7 +407,7 @@ bool COptions::GetXmlValue(unsigned int nID, wxString &value, TiXmlElement *sett
 		settings = m_pXmlFile->GetElement()->FirstChildElement("Settings");
 		if (!settings)
 		{
-			TiXmlNode *node = m_pXmlFile->GetElement()->InsertEndChild(TiXmlElement("Settings"));
+			TiXmlNode *node = m_pXmlFile->GetElement()->LinkEndChild(new TiXmlElement("Settings"));
 			if (!node)
 				return false;
 			settings = node->ToElement();
@@ -515,6 +519,11 @@ int COptions::Validate(unsigned int nID, int value)
 
 wxString COptions::Validate(unsigned int nID, wxString value)
 {
+	if (nID == OPTION_INVALID_CHAR_REPLACE)
+	{
+		if (value.Len() > 1)
+			value = _T("_");
+	}
 	return value;
 }
 
@@ -554,7 +563,7 @@ void COptions::SetServer(wxString path, const CServer& server)
 			char *utf8 = ConvUTF8(sub);
 			if (!utf8)
 				return;
-			TiXmlNode *node = element->InsertEndChild(TiXmlElement(utf8));
+			TiXmlNode *node = element->LinkEndChild(new TiXmlElement(utf8));
 			delete [] utf8;
 			if (!node || !node->ToElement())
 				return;
