@@ -76,6 +76,10 @@ enum interfaceOptions
 	OPTION_EDIT_TRACK_LOCAL,
 	OPTION_PREVENT_IDLESLEEP,
 	OPTION_FILTEREDIT_SIZE,
+	OPTION_INVALID_CHAR_REPLACE_ENABLE,
+	OPTION_INVALID_CHAR_REPLACE,
+	OPTION_ALREADYCONNECTED_CHOICE,
+	OPTION_EDITSTATUSDIALOG_SIZE,
 
 	// Has to be last element
 	OPTIONS_NUM
