@@ -26,6 +26,9 @@
 #include "recursive_operation.h"
 #include "auto_ascii_files.h"
 #include "dragdropmanager.h"
+#if WITH_LIBDBUS
+#include "../dbus/desktop_notification.h"
+#endif
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -60,7 +63,9 @@ public:
 
 		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
 		{
-			CState* const pState = m_pQueueView->m_pMainFrame->GetState();
+			CState* const pState = CContextManager::Get()->GetCurrentContext();
+			if (!pState)
+				return wxDragNone;
 			const CServer* const pServer = pState->GetServer();
 			if (!pServer)
 				return wxDragNone;
@@ -79,7 +84,9 @@ public:
 				return wxDragNone;
 			}
 
-			CState* const pState = m_pQueueView->m_pMainFrame->GetState();
+			CState* const pState = CContextManager::Get()->GetCurrentContext();
+			if (!pState)
+				return wxDragNone;
 			const CServer* const pServer = pState->GetServer();
 			if (!pServer)
 				return wxDragNone;
@@ -122,7 +129,9 @@ public:
 		if (pDragDropManager && !pDragDropManager->remoteParent.IsEmpty())
 		{
 			// Drag from remote to queue, check if local path is writeable
-			CState* const pState = m_pQueueView->m_pMainFrame->GetState();
+			CState* const pState = CContextManager::Get()->GetCurrentContext();
+			if (!pState)
+				return wxDragNone;
 			if (!pState->GetLocalDir().IsWriteable())
 				return wxDragNone;			
 		}
@@ -486,6 +495,10 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 #endif
 
 	m_resize_timer.SetOwner(this);
+
+#if WITH_LIBDBUS
+	m_desktop_notification = 0;
+#endif
 }
 
 CQueueView::~CQueueView()
@@ -500,6 +513,10 @@ CQueueView::~CQueueView()
 	DeleteEngines();
 
 	m_resize_timer.Stop();
+
+#if WITH_LIBDBUS
+	delete m_desktop_notification;
+#endif
 }
 
 bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxString& localFile,
@@ -544,7 +561,7 @@ void CQueueView::QueueFile_Finish(const bool start)
 	if (!m_activeMode && start)
 	{
 		m_activeMode = 1;
-		m_pMainFrame->GetState()->NotifyHandlers(STATECHANGE_QUEUEPROCESSING);
+		CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_QUEUEPROCESSING);
 	}
 
 	if (m_activeMode)
@@ -572,7 +589,7 @@ bool CQueueView::QueueFiles(const bool queueOnly, const wxString& localPath, con
 			continue;
 
 		CFileItem* fileItem;
-		fileItem = new CFileItem(pServerItem, queueOnly, true, localPath + iter->name, iter->name, dataObject.GetServerPath(), iter->size);
+		fileItem = new CFileItem(pServerItem, queueOnly, true, localPath + ReplaceInvalidCharacters(iter->name), iter->name, dataObject.GetServerPath(), iter->size);
 		fileItem->m_transferSettings.binary = !CAutoAsciiFiles::TransferRemoteAsAscii(iter->name, dataObject.GetServerPath().GetType());
 
 		InsertItem(pServerItem, fileItem);
@@ -710,12 +727,11 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, CNotification* p
 		delete pNotification;
 		break;
 	case nId_local_dir_created:
-		if (m_pMainFrame)
 		{
 			CLocalDirCreatedNotification *pLocalDirCreatedNotification = reinterpret_cast<CLocalDirCreatedNotification *>(pNotification);
-			CState* pState = m_pMainFrame->GetState();
-			if (pState)
-				pState->LocalDirCreated(pLocalDirCreatedNotification->dir);
+			const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+			for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+				(*iter)->LocalDirCreated(pLocalDirCreatedNotification->dir);
 		}
 		delete pNotification;
 		break;
@@ -725,6 +741,65 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, CNotification* p
 	}
 }
 
+bool CQueueView::CanStartTransfer(const CServerItem& server_item, struct t_EngineData *&pEngineData)
+{
+	const CServer &server = server_item.GetServer();
+	const int max_count = server.MaximumMultipleConnections();
+	if (!max_count)
+		return true;
+
+	int active_count = server_item.m_activeCount;
+
+	bool browsing_on_same = false;
+	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+	CState* pState = 0;
+	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+	{
+		pState = *iter;
+		const CServer* pBrowsingServer = pState->GetServer();
+		if (!pBrowsingServer)
+			continue;
+
+		if (*pBrowsingServer == server)
+		{
+			active_count++;
+			browsing_on_same = true;
+			break;
+		}
+	}
+
+	if (active_count < max_count)
+		return true;
+
+	// Max count has been reached
+
+	// If we got an idle engine connected to this very server, start the
+	// transfer anyhow. Let's not get this connection go to waste.
+	pEngineData = GetIdleEngine(&server);
+	if (pEngineData->pEngine->IsConnected() && pEngineData->lastServer == server)
+		return true;
+
+	if (!browsing_on_same || active_count > 1)
+		return false;
+
+	// At this point the following holds:
+	// max_count is limited to 1, only connection to server is held by the browsing connection
+	pEngineData = m_engineData[0];
+
+	if (pEngineData->active)
+		return false;
+
+	if (pEngineData->pEngine != pState->m_pEngine)
+	{
+		if (pEngineData->pEngine)
+			ReleaseExclusiveEngineLock(pEngineData->pEngine);
+		pEngineData->state = t_EngineData::waitprimary;
+		pEngineData->pEngine = pState->m_pEngine;
+	}
+
+	return true;
+}
+
 bool CQueueView::TryStartNextTransfer()
 {
 	if (m_quit || !m_activeMode)
@@ -764,48 +839,9 @@ bool CQueueView::TryStartNextTransfer()
 	{
 		t_EngineData* pEngineData = 0;
 		CServerItem* currentServerItem = *iter;
-		int maxCount = currentServerItem->GetServer().MaximumMultipleConnections();
-		int activeCount = currentServerItem->m_activeCount;
-		const CServer* pBrowsingServer = m_pMainFrame->GetState()->GetServer();
-		bool browsingOnSame = false;
-		if (pBrowsingServer && currentServerItem->GetServer() == *pBrowsingServer)
-		{
-			browsingOnSame = true;
-			activeCount++;
-		}
-		if (maxCount && activeCount >= maxCount)
-		{
-			// If we got an idle engine connected to this very server, start the
-			// transfer anyhow. Let's not get this connection go to waste.
-			pEngineData = GetIdleEngine(&(*iter)->GetServer());
-			if (pEngineData)
-			{
-				if (!pEngineData->pEngine->IsConnected() || pEngineData->lastServer != (*iter)->GetServer())
-				{
-					// If the browsing connection is connected to this server and idle, use it
-					if (browsingOnSame && activeCount == 1)
-					{
-						pEngineData = m_engineData[0];
-						if (pEngineData->active)
-							continue;
-					}
-					else
-						continue;
-				}
-			}
-			else
-			{
-				// If the browsing connection is connected to this server and idle, use it
-				if (browsingOnSame && activeCount == 1)
-				{
-					pEngineData = m_engineData[0];
-					if (pEngineData->active)
-						continue;
-				}
-				else
-					continue;
-			}
-		}
+
+		if (!CanStartTransfer(*currentServerItem, pEngineData))
+			continue;
 
 		CFileItem* newFileItem = currentServerItem->GetIdleChild(m_activeMode == 1, wantedDirection);
 
@@ -813,8 +849,9 @@ bool CQueueView::TryStartNextTransfer()
 		{
 			wxFileName fn(newFileItem->GetLocalFile(), _T(""));
 			wxFileName::Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
-			CState* const pState = m_pMainFrame->GetState();
-			pState->RefreshLocalFile(fn.GetFullPath());
+			const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+			for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+				(*iter)->RefreshLocalFile(fn.GetFullPath());
 			if (RemoveItem(newFileItem, true))
 			{
 				// Server got deleted. Unfortunately we have to start over now
@@ -872,28 +909,27 @@ bool CQueueView::TryStartNextTransfer()
 	const CServer oldServer = pEngineData->lastServer;
 	pEngineData->lastServer = bestMatch.serverItem->GetServer();
 
-	if (!pEngineData->pEngine)
+	if (pEngineData->state != t_EngineData::waitprimary)
 	{
-		pEngineData->state = t_EngineData::waitprimary;
-	}
-	else if (!pEngineData->pEngine->IsConnected())
-	{
-		if (pEngineData->lastServer.GetLogonType() == ASK)
+		if (!pEngineData->pEngine->IsConnected())
 		{
-			if (CLoginManager::Get().GetPassword(pEngineData->lastServer, true))
-				pEngineData->state = t_EngineData::connect;
+			if (pEngineData->lastServer.GetLogonType() == ASK)
+			{
+				if (CLoginManager::Get().GetPassword(pEngineData->lastServer, true))
+					pEngineData->state = t_EngineData::connect;
+				else
+					pEngineData->state = t_EngineData::askpassword;
+			}
 			else
-				pEngineData->state = t_EngineData::askpassword;
+				pEngineData->state = t_EngineData::connect;
 		}
+		else if (oldServer != bestMatch.serverItem->GetServer())
+			pEngineData->state = t_EngineData::disconnect;
+		else if (pEngineData->pItem->GetType() == QueueItemType_File)
+			pEngineData->state = t_EngineData::transfer;
 		else
-			pEngineData->state = t_EngineData::connect;
+			pEngineData->state = t_EngineData::mkdir;
 	}
-	else if (oldServer != bestMatch.serverItem->GetServer())
-		pEngineData->state = t_EngineData::disconnect;
-	else if (pEngineData->pItem->GetType() == QueueItemType_File)
-		pEngineData->state = t_EngineData::transfer;
-	else
-		pEngineData->state = t_EngineData::mkdir;
 
 	if (bestMatch.fileItem->GetType() == QueueItemType_File)
 	{
@@ -1176,7 +1212,11 @@ void CQueueView::ResetEngine(t_EngineData& data, const enum ResetReason reason)
 
 			CFileItem* const pFileItem = (CFileItem*)data.pItem;
 			if (pFileItem->Download())
-				m_pMainFrame->GetState()->RefreshLocalFile(pFileItem->GetLocalFile());
+			{
+				const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+				for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+					(*iter)->RefreshLocalFile(pFileItem->GetLocalFile());
+			}
 
 			if (pFileItem->m_edit != CEditHandler::none && reason != retry && reason != reset)
 			{
@@ -1271,11 +1311,19 @@ void CQueueView::ResetEngine(t_EngineData& data, const enum ResetReason reason)
 		m_activeCount--;
 	data.active = false;
 
-	if (data.state == t_EngineData::waitprimary)
+	if (data.state == t_EngineData::waitprimary && data.pEngine)
 	{
-		CCommandQueue* pCommandQueue = m_pMainFrame->GetState()->m_pCommandQueue;
-		if (pCommandQueue)
-			pCommandQueue->RequestExclusiveEngine(false);
+		const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+		for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+		{
+			CState* pState = *iter;
+			if (pState->m_pEngine != data.pEngine)
+				continue;
+			CCommandQueue* pCommandQueue = pState->m_pCommandQueue;
+			if (pCommandQueue)
+				pCommandQueue->RequestExclusiveEngine(false);
+			break;
+		}
 	}
 
 	data.state = t_EngineData::none;
@@ -1325,7 +1373,29 @@ void CQueueView::SendNextCommand(t_EngineData& engineData)
 		if (engineData.state == t_EngineData::waitprimary)
 		{
 			engineData.pItem->m_statusMessage = _("Waiting for browsing connection");
-			CState* const pState = m_pMainFrame->GetState();
+
+			wxASSERT(engineData.pEngine);
+			if (!engineData.pEngine)
+			{
+				ResetEngine(engineData, retry);
+				return;
+			}
+
+			CState* pState = 0;
+			const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+			for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+			{
+				if ((*iter)->m_pEngine != engineData.pEngine)
+					continue;
+				pState = *iter;
+				break;
+			}
+			if (!pState)
+			{
+				ResetEngine(engineData, retry);
+				return;
+			}
+
 			CCommandQueue* pCommandQueue = pState->m_pCommandQueue;
 			pCommandQueue->RequestExclusiveEngine(true);
 			return;
@@ -1473,17 +1543,19 @@ bool CQueueView::SetActive(bool active /*=true*/)
 		for (std::vector<CServerItem*>::iterator iter = m_serverList.begin(); iter != m_serverList.end(); iter++)
 			(*iter)->QueueImmediateFiles();
 
-		CState* const pState = m_pMainFrame->GetState();
-		if (pState)
+		const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+		for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
 		{
+			CState* pState = *iter;
+
 			CRecursiveOperation* pRecursiveOperation = pState->GetRecursiveOperationHandler();
-			if (pRecursiveOperation)
-			{
-				if (pRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_download)
-					pRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue);
-				if (pRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_download_flatten)
-					pRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue_flatten);
-			}
+			if (!pRecursiveOperation)
+				continue;
+
+			if (pRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_download)
+				pRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue);
+			if (pRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_download_flatten)
+				pRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue_flatten);
 		}
 
 		UpdateStatusLinePositions();
@@ -1511,7 +1583,7 @@ bool CQueueView::SetActive(bool active /*=true*/)
 			}
 		}
 
-		m_pMainFrame->GetState()->NotifyHandlers(STATECHANGE_QUEUEPROCESSING);
+		CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_QUEUEPROCESSING);
 
 		return m_activeCount == 0;
 	}
@@ -1525,7 +1597,7 @@ bool CQueueView::SetActive(bool active /*=true*/)
 		UpdateStatusLinePositions();
 	}
 
-	m_pMainFrame->GetState()->NotifyHandlers(STATECHANGE_QUEUEPROCESSING);
+	CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_QUEUEPROCESSING);
 
 	return true;
 }
@@ -1583,9 +1655,7 @@ void CQueueView::CheckQueueState()
 {
 	if (!m_engineData.empty() && !m_engineData[0]->active && m_engineData[0]->pEngine)
 	{
-		CCommandQueue* pCommandQueue = m_pMainFrame->GetState()->m_pCommandQueue;
-		if (pCommandQueue)
-			pCommandQueue->ReleaseEngine();
+		ReleaseExclusiveEngineLock(m_engineData[0]->pEngine);
 		m_engineData[0]->pEngine = 0;
 	}
 
@@ -1610,7 +1680,7 @@ void CQueueView::CheckQueueState()
 
 		TryRefreshListings();
 
-		m_pMainFrame->GetState()->NotifyHandlers(STATECHANGE_QUEUEPROCESSING);
+		CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_QUEUEPROCESSING);
 
 		if (!m_quit)
 			ActionAfter();
@@ -2528,53 +2598,64 @@ void CQueueView::TryRefreshListings()
 	if (m_quit)
 		return;
 
-	const CState* const pState = m_pMainFrame->GetState();
-	const CServer* const pServer = pState->GetServer();
-	if (!pServer)
-		return;
-
-	const CDirectoryListing* const pListing = pState->GetRemoteDir().Value();
-	if (!pListing)
-		return;
-
-	// See if there's an engine that is already listing
-	for (unsigned int i = 1; i < m_engineData.size(); i++)
+	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
 	{
-		if (!m_engineData[i]->active || m_engineData[i]->state != t_EngineData::list)
+		CState* pState = *iter;
+
+		const CServer* const pServer = pState->GetServer();
+		if (!pServer)
 			continue;
 
-		if (m_engineData[i]->lastServer == *pServer)
+		const CDirectoryListing* const pListing = pState->GetRemoteDir().Value();
+		if (!pListing)
+			continue;
+
+		// See if there's an engine that is already listing
+		unsigned int i;
+		for (i = 1; i < m_engineData.size(); i++)
 		{
+			if (!m_engineData[i]->active || m_engineData[i]->state != t_EngineData::list)
+				continue;
+
+			if (m_engineData[i]->lastServer != *pServer)
+				continue;
+
 			// This engine is already listing a directory on the current server
-			return;
+			break;
 		}
+		if (i != m_engineData.size())
+			continue;
+
+		if (m_last_refresh_server == *pServer && m_last_refresh_path == pListing->path &&
+			m_last_refresh_listing_time == pListing->m_firstListTime)
+		{
+			// Do not try to refresh same directory multiple times
+			continue;
+		}
+
+		t_EngineData* pEngineData = GetIdleEngine(pServer);
+		if (!pEngineData)
+			continue;
+
+		if (!pEngineData->pEngine->IsConnected() || pEngineData->lastServer != *pServer)
+			continue;
+
+		m_last_refresh_server = *pServer;
+		m_last_refresh_path = pListing->path;
+		m_last_refresh_listing_time = pListing->m_firstListTime;
+
+		CListCommand command(pListing->path, _T(""), LIST_FLAG_AVOID);
+		int res = pEngineData->pEngine->Command(command);
+		if (res != FZ_REPLY_WOULDBLOCK)
+			continue;
+
+		pEngineData->active = true;
+		pEngineData->state = t_EngineData::list;
+		m_activeCount++;
+
+		break;
 	}
-
-	if (m_last_refresh_server == *pServer && m_last_refresh_path == pListing->path &&
-		m_last_refresh_listing_time == pListing->m_firstListTime)
-	{
-		// Do not try to refresh same directory multiple times
-		return;
-	}
-	m_last_refresh_server = *pServer;
-	m_last_refresh_path = pListing->path;
-	m_last_refresh_listing_time = pListing->m_firstListTime;
-	
-	t_EngineData* pEngineData = GetIdleEngine(pServer);
-	if (!pEngineData)
-		return;
-
-	if (!pEngineData->pEngine->IsConnected() || pEngineData->lastServer != *pServer)
-		return;
-
-	CListCommand command(pListing->path, _T(""), LIST_FLAG_AVOID);
-	int res = pEngineData->pEngine->Command(command);
-	if (res != FZ_REPLY_WOULDBLOCK)
-		return;
-
-	pEngineData->active = true;
-	pEngineData->state = t_EngineData::list;
-	m_activeCount++;
 }
 
 void CQueueView::OnAskPassword(wxCommandEvent& event)
@@ -2781,7 +2862,7 @@ void CQueueView::WriteToFile(TiXmlElement* pElement) const
 	TiXmlElement* pQueue = pElement->FirstChildElement("Queue");
 	if (!pQueue)
 	{
-		pQueue = pElement->InsertEndChild(TiXmlElement("Queue"))->ToElement();
+		pQueue = pElement->LinkEndChild(new TiXmlElement("Queue"))->ToElement();
 	}
 
 	wxASSERT(pQueue);
@@ -2836,40 +2917,56 @@ void CQueueView::OnSetPriority(wxCommandEvent& event)
 
 void CQueueView::OnExclusiveEngineRequestGranted(wxCommandEvent& event)
 {
-	CCommandQueue* pCommandQueue = m_pMainFrame->GetState()->m_pCommandQueue;
-	if (!pCommandQueue)
-		return;
-
-	CFileZillaEngine* pEngine = pCommandQueue->GetEngineExclusive(event.GetId());
-
 	t_EngineData* pEngineData = m_engineData[0];
 
-	if (!pEngineData->active)
+	CFileZillaEngine* pEngine = 0;
+	CState* pState;
+	CCommandQueue* pCommandQueue;
+	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
 	{
-		wxASSERT(!pEngineData->pEngine);
-		if (pEngine)
-			pCommandQueue->ReleaseEngine();
+		pState = *iter;
+		pCommandQueue = pState->m_pCommandQueue;
+		if (!pCommandQueue)
+			continue;
 
-		return;
+		pEngine = pCommandQueue->GetEngineExclusive(event.GetId());
+		if (!pEngine)
+			continue;
+
+		if (pEngine != pEngineData->pEngine)
+		{
+			pCommandQueue->ReleaseEngine();
+			pEngine = 0;
+			continue;
+		}
+
+		break;
 	}
 
 	if (!pEngine)
 		return;
 
-	wxASSERT(!pEngineData->pEngine);
+	if (!pEngineData->active)
+	{
+		pCommandQueue->ReleaseEngine();
+		return;
+	}
 
 	wxASSERT(pEngineData->state == t_EngineData::waitprimary);
 	if (pEngineData->state != t_EngineData::waitprimary)
 		return;
 
-	const CServer* pCurrentServer = m_pMainFrame->GetState()->GetServer();
 	CServerItem* pServerItem = (CServerItem*)pEngineData->pItem->GetParent();
 
+	const CServer* pCurrentServer = pState->GetServer();
+
 	wxASSERT(pServerItem);
 
 	if (!pCurrentServer || *pCurrentServer != pServerItem->GetServer())
 	{
-		pCommandQueue->ReleaseEngine();
+		if (pState->m_pCommandQueue)
+			pState->m_pCommandQueue->ReleaseEngine();
 		ResetEngine(*pEngineData, retry);
 		return;
 	}
@@ -2896,6 +2993,39 @@ bool CQueueView::IsActionAfter(enum ActionAfterState state)
 
 void CQueueView::ActionAfter(bool warned /*=false*/)
 {
+	// Need to check all contexts whether there's a recursive
+	// download operation still in progress
+	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+	for (unsigned int i = 0; i < pStates->size(); i++)
+	{
+		CState *pState = (*pStates)[i];
+		CRecursiveOperation *pRecursiveOperationHandler;
+		if (!pState || !(pRecursiveOperationHandler = pState->GetRecursiveOperationHandler()))
+			continue;
+
+		if (pRecursiveOperationHandler->GetOperationMode() == CRecursiveOperation::recursive_download ||
+			pRecursiveOperationHandler->GetOperationMode() == CRecursiveOperation::recursive_download_flatten)
+		{
+			return;
+		}
+	}
+
+#if WITH_LIBDBUS
+	if (!m_pMainFrame->IsActive())
+	{
+		if (!m_desktop_notification)
+			m_desktop_notification = new CDesktopNotification;
+		int failed_count = m_pQueue->GetQueueView_Failed()->GetFileCount();
+		if (failed_count != 0)
+		{
+			wxString fmt = wxPLURAL("All transfers have finished. %d file could not be transferred.", "All transfers have finished. %d files could not be transferred.", failed_count);
+			m_desktop_notification->Notify(_("Transfers finished"), wxString::Format(fmt, failed_count), _T("transfer.error"));
+		}
+		else
+			m_desktop_notification->Notify(_("Transfers finished"), _("All files have been successfully transferred"), _T("transfer.complete"));
+	}
+#endif
+
 	switch (m_actionAfterState)
 	{
 		case ActionAfterState_Close:
@@ -2905,8 +3035,13 @@ void CQueueView::ActionAfter(bool warned /*=false*/)
 		}
 		case ActionAfterState_Disconnect:
 		{
-			if (m_pMainFrame->GetState()->IsRemoteConnected() && m_pMainFrame->GetState()->IsRemoteIdle())
-				m_pMainFrame->GetState()->m_pCommandQueue->ProcessCommand(new CDisconnectCommand());
+			const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+			for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+			{
+				CState* pState = *iter;
+				if (pState->IsRemoteConnected() && pState->IsRemoteIdle())
+					pState->Disconnect();
+			}
 			break;
 		}
 		case ActionAfterState_RunCommand:
@@ -3187,3 +3322,65 @@ void CQueueView::RenameFileInTransfer(CFileZillaEngine *pEngine, const wxString&
 
 	RefreshItem(pFile);
 }
+
+wxString CQueueView::ReplaceInvalidCharacters(const wxString& filename)
+{
+	if (!COptions::Get()->GetOptionVal(OPTION_INVALID_CHAR_REPLACE_ENABLE))
+		return filename;
+
+	const wxChar replace = COptions::Get()->GetOption(OPTION_INVALID_CHAR_REPLACE)[0];
+
+	wxString result;
+
+	wxChar* buf = result.GetWriteBuf(filename.Len() + 1);
+
+	const wxChar* p = filename.c_str();
+	while (*p)
+	{
+		switch (*p)
+		{
+		case '/':
+#ifdef __WXMSW__
+		case '\\':
+		case ':':
+		case '*':
+		case '?':
+		case '"':
+		case '<':
+		case '>':
+		case '|':
+#endif
+			if (replace)
+				*buf++ = replace;
+			break;
+		default:
+			*buf++ = *p;
+		}
+		p++;
+	}
+	*buf = 0;
+
+	result.UngetWriteBuf();
+
+	return result;
+}
+
+void CQueueView::ReleaseExclusiveEngineLock(CFileZillaEngine* pEngine)
+{
+	wxASSERT(pEngine);
+	if (!pEngine)
+		return;
+
+	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
+	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
+	{
+		CState* pState = *iter;
+		if (pState->m_pEngine != pEngine)
+			continue;
+		CCommandQueue *pCommandQueue = pState->m_pCommandQueue;
+		if (pCommandQueue)
+			pCommandQueue->ReleaseEngine();
+
+		break;
+	}
+}
