@@ -76,6 +76,9 @@ class CStatusLineCtrl;
 class CFolderProcessingThread;
 class CAsyncRequestQueue;
 class CQueue;
+#if WITH_LIBDBUS
+class CDesktopNotification;
+#endif
 
 class CQueueView : public CQueueViewBase
 {
@@ -123,10 +126,17 @@ public:
 
 	void RenameFileInTransfer(CFileZillaEngine *pEngine, const wxString& newName, bool local);
 
+	static wxString ReplaceInvalidCharacters(const wxString& filename);
+
 protected:
 
 	void AdvanceQueue(bool refresh = true);
 	bool TryStartNextTransfer();
+
+	// Called from TryStartNextTransfer(), checks
+	// whether it is allowed to start another transfer on that server item
+	bool CanStartTransfer(const CServerItem& server_item, struct t_EngineData *&pEngineData);
+
 	bool ProcessFolderItems(int type = -1);
 	void ProcessUploadFolderItems();
 
@@ -243,6 +253,12 @@ protected:
 
 	wxTimer m_resize_timer;
 
+	void ReleaseExclusiveEngineLock(CFileZillaEngine* pEngine);
+
+#if WITH_LIBDBUS
+	CDesktopNotification* m_desktop_notification;
+#endif
+
 	DECLARE_EVENT_TABLE();
 	void OnEngineEvent(wxEvent &event);
 	void OnFolderThreadComplete(wxCommandEvent& event);
