@@ -149,7 +149,7 @@ public:
 				dir.AddSegment(info.name);
 				if (dir == target || dir.IsParentOf(target, false))
 				{
-					wxMessageBox(_("A directory cannot be dragged into one if its subdirectories."));
+					wxMessageBox(_("A directory cannot be dragged into one of its subdirectories."));
 					return wxDragNone;
 				}
 			}
@@ -337,11 +337,11 @@ BEGIN_EVENT_TABLE(CRemoteListView, CFileListCtrl<CGenericFileData>)
 	EVT_MENU(XRCID("ID_EDIT"), CRemoteListView::OnMenuEdit)
 	EVT_MENU(XRCID("ID_ENTER"), CRemoteListView::OnMenuEnter)
 	EVT_MENU(XRCID("ID_GETURL"), CRemoteListView::OnMenuGeturl)
+	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CRemoteListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
 CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView* pQueue)
 	: CFileListCtrl<CGenericFileData>(pParent, pState, pQueue),
-	CSystemImageList(16),
 	CStateEventHandler(pState)
 {
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
@@ -356,7 +356,7 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView*
 
 	const unsigned long widths[6] = { 80, 75, 80, 100, 80, 80 };
 
-	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0]);
+	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0], true);
 	AddColumn(_("Filesize"), wxLIST_FORMAT_RIGHT, widths[1]);
 	AddColumn(_("Filetype"), wxLIST_FORMAT_LEFT, widths[2]);
 	AddColumn(_("Last modified"), wxLIST_FORMAT_LEFT, widths[3]);
@@ -369,9 +369,7 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView*
 	m_dirIcon = GetIconIndex(dir);
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
-#ifdef __WXMSW__
 	InitHeaderImageList();
-#endif
 
 	SetDirectoryListing(0);
 
@@ -405,7 +403,7 @@ int CRemoteListView::OnGetItemImage(long item) const
 	if (icon != -2)
 		return icon;
 
-	icon = pThis->GetIconIndex(file, (*m_pDirectoryListing)[index].name, false);
+	icon = pThis->GetIconIndex(file, (*m_pDirectoryListing)[index].name, false, (*m_pDirectoryListing)[index].dir);
 	return icon;
 }
 
@@ -453,7 +451,16 @@ void CRemoteListView::UpdateDirectoryListing_Added(const CSharedPointer<const CD
 		const CDirentry& entry = (*pDirectoryListing)[i];
 		CGenericFileData data;
 		data.flags = normal;
-		data.icon = entry.dir ? m_dirIcon : -2;
+		if (entry.dir)
+		{
+			data.icon = m_dirIcon;
+#ifndef __WXMSW__
+			if (entry.link)
+				data.icon += 3;
+#endif
+		}
+		else
+			data.icon = -2;
 		m_fileData.push_back(data);
 
 		if (filter.FilenameFiltered(entry.name, path, entry.dir, entry.size, false, 0))
@@ -515,7 +522,7 @@ void CRemoteListView::UpdateDirectoryListing_Added(const CSharedPointer<const CD
 	if (m_pFilelistStatusBar)
 		m_pFilelistStatusBar->SetHidden(m_pDirectoryListing->GetCount() + 1 - m_indexMapping.size());
 
-	wxASSERT(m_indexMapping.size() == pDirectoryListing->GetCount() + 1);
+	wxASSERT(m_indexMapping.size() <= pDirectoryListing->GetCount() + 1);
 }
 
 void CRemoteListView::UpdateDirectoryListing_Removed(const CSharedPointer<const CDirectoryListing> &pDirectoryListing)
@@ -780,7 +787,16 @@ void CRemoteListView::SetDirectoryListing(const CSharedPointer<const CDirectoryL
 			const CDirentry& entry = (*m_pDirectoryListing)[i];
 			CGenericFileData data;
 			data.flags = normal;
-			data.icon = entry.dir ? m_dirIcon : -2;
+			if (entry.dir)
+			{
+				data.icon = m_dirIcon;
+#ifndef __WXMSW__
+				if (entry.link)
+					data.icon += 3;
+#endif
+			}
+			else
+				data.icon = -2;
 			m_fileData.push_back(data);
 
 			if (filter.FilenameFiltered(entry.name, path, entry.dir, entry.size, false, 0))
@@ -1358,6 +1374,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
 		pMenu->Enable(XRCID("ID_GETURL"), false);
+		pMenu->Enable(XRCID("ID_CONTEXT_REFRESH"), false);
 	}
 	else if ((GetItemCount() && GetItemState(0, wxLIST_STATE_SELECTED)))
 	{
@@ -1515,7 +1532,7 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 			if (!idle)
 				continue;
 			CLocalPath local_path(local_parent);
-			local_path.AddSegment(name);
+			local_path.AddSegment(CQueueView::ReplaceInvalidCharacters(name));
 			CServerPath remotePath = m_pDirectoryListing->path;
 			if (remotePath.AddSegment(name))
 			{
@@ -1525,7 +1542,7 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 		}
 		else
 		{
-			m_pQueue->QueueFile(queueOnly, true, local_parent.GetPath() + name, name, m_pDirectoryListing->path, *pServer, entry.size);
+			m_pQueue->QueueFile(queueOnly, true, local_parent.GetPath() + CQueueView::ReplaceInvalidCharacters(name), name, m_pDirectoryListing->path, *pServer, entry.size);
 			added = true;
 		}
 	}
@@ -3115,3 +3132,25 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
 	wxTheClipboard->Flush();
 	wxTheClipboard->Close();
 }
+
+#ifdef __WXMSW__
+int CRemoteListView::GetOverlayIndex(int item)
+{
+	int index = GetItemIndex(item);
+	if (index == -1)
+		return 0;
+	if ((unsigned int)index >= m_pDirectoryListing->GetCount())
+		return 0;
+
+	if ((*m_pDirectoryListing)[index].link)
+		return GetLinkOverlayIndex();
+
+	return 0;
+}
+#endif
+
+void CRemoteListView::OnMenuRefresh(wxCommandEvent& event)
+{
+	if (m_pState)
+		m_pState->RefreshRemote();
+}
