@@ -10,7 +10,7 @@ class CChmodDialog;
 class CInfoText;
 class CRemoteListViewDropTarget;
 
-class CRemoteListView : public CFileListCtrl<CGenericFileData>, CSystemImageList, CStateEventHandler
+class CRemoteListView : public CFileListCtrl<CGenericFileData>, CStateEventHandler
 {
 	friend class CRemoteListViewDropTarget;
 	friend class CRemoteListViewSortType;
@@ -80,6 +80,10 @@ protected:
 	void RepositionInfoText();
 	void SetInfoText();
 
+#ifdef __WXMSW__
+	virtual int GetOverlayIndex(int item);
+#endif
+
 	wxDropTarget* m_pDropTarget;
 	int m_dropTarget;
 
@@ -113,6 +117,7 @@ protected:
 	void OnMenuEdit(wxCommandEvent& event);
 	void OnMenuEnter(wxCommandEvent& event);
 	void OnMenuGeturl(wxCommandEvent& event);
+	void OnMenuRefresh(wxCommandEvent& event);
 };
 
 #endif
