@@ -8,6 +8,8 @@
 #include "inputdialog.h"
 #include "dragdropmanager.h"
 #include <wx/clipbrd.h>
+#include "queue.h"
+#include "QueueView.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -109,7 +111,7 @@ public:
 					dir.AddSegment(info.name);
 					if (dir == path || dir.IsParentOf(path, false))
 					{
-						wxMessageBox(_("A directory cannot be dragged into one if its subdirectories."));
+						wxMessageBox(_("A directory cannot be dragged into one of its subdirectories."));
 						return wxDragNone;
 					}
 				}
@@ -208,9 +210,9 @@ protected:
 	wxDataObjectComposite* m_pDataObject;
 };
 
-IMPLEMENT_CLASS(CRemoteTreeView, wxTreeCtrl)
+IMPLEMENT_CLASS(CRemoteTreeView, wxTreeCtrlEx)
 
-BEGIN_EVENT_TABLE(CRemoteTreeView, wxTreeCtrl)
+BEGIN_EVENT_TABLE(CRemoteTreeView, wxTreeCtrlEx)
 EVT_TREE_ITEM_EXPANDING(wxID_ANY, CRemoteTreeView::OnItemExpanding)
 EVT_TREE_SEL_CHANGED(wxID_ANY, CRemoteTreeView::OnSelectionChanged)
 EVT_TREE_ITEM_ACTIVATED(wxID_ANY, CRemoteTreeView::OnItemActivated)
@@ -232,7 +234,7 @@ EVT_MENU(XRCID("ID_GETURL"), CRemoteTreeView::OnMenuGeturl)
 END_EVENT_TABLE()
 
 CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState, CQueueView* pQueue)
-	: wxTreeCtrl(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER | wxTR_HIDE_ROOT),
+	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER | wxTR_HIDE_ROOT),
 	CSystemImageList(16),
 	CStateEventHandler(pState)
 {
@@ -347,7 +349,7 @@ void CRemoteTreeView::SetDirectoryListing(const CSharedPointer<const CDirectoryL
 	m_freezeCount--;
 #endif
 	if (!modified)
-		SelectItem(parent);
+		SafeSelectItem(parent);
 #ifndef __WXMSW__
 	else
 		Refresh();
@@ -1089,7 +1091,7 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 
 	const wxString& name = GetItemText(m_contextMenuItem);
 
-	localDir.AddSegment(name);
+	localDir.AddSegment(CQueueView::ReplaceInvalidCharacters(name));
 
 	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
 	pRecursiveOperation->AddDirectoryToVisit(path, _T(""), localDir.GetPath());
