@@ -17,12 +17,19 @@ public:
 
 	void SetQueue(CQueueView *pQueue);
 
+	void TriggerProcessing();
+
 protected:
+
+	// Returns falls if main window doesn't have focus or is minimized.
+	// Request attention if needed
+	bool CheckWindowState();
+
 	CMainFrame *m_pMainFrame;
 	CQueueView *m_pQueueView;
 	CVerifyCertDialog *m_pVerifyCertDlg;
 
-	void ProcessNextRequest();
+	bool ProcessNextRequest();
 	bool ProcessDefaults(CFileZillaEngine *pEngine, CAsyncRequestNotification *pNotification);
 
 	struct t_queueEntry
@@ -34,6 +41,9 @@ protected:
 
 	DECLARE_EVENT_TABLE();
 	void OnProcessQueue(wxCommandEvent &event);
+
+	// Reentrancy guard
+	bool m_inside_request;
 };
 
 #endif //__ASYNCREQUESTQUEUE_H__
