@@ -4,6 +4,7 @@
 #include "filezillaapp.h"
 #include "queue.h"
 #include "Options.h"
+#include "window_state_manager.h"
 
 // Defined in optionspage_edit.cpp
 bool UnquoteCommand(wxString& command, wxString& arguments);
@@ -651,7 +652,7 @@ checkmodifications_remote:
 
 		// File has changed, ask user what to do
 
-        wxTopLevelWindow* pTopWindow = (wxTopLevelWindow*)wxTheApp->GetTopWindow();
+		wxTopLevelWindow* pTopWindow = (wxTopLevelWindow*)wxTheApp->GetTopWindow();
 		if (pTopWindow && pTopWindow->IsIconized())
 		{
 			pTopWindow->RequestUserAttention(wxUSER_ATTENTION_INFO);
@@ -912,7 +913,7 @@ wxString CEditHandler::GetOpenCommand(const wxString& file, bool& program_exists
 	else if (command[0] == '1')
 	{
 		// Text editor
-		const wxString random = _T("5AC2EE515D18406aB77C2C60F1F88952.txt"); // Chosen by fair dice roll. Guaranteed to be random.
+		const wxString random = _T("5AC2EE515D18406 space aB77C2C60F1F88952.txt"); // Chosen by fair dice roll. Guaranteed to be random.
 		wxString command = GetSystemOpenCommand(random, program_exists);
 		if (command.empty() || !program_exists)
 			return command;
@@ -978,7 +979,7 @@ static bool PathExpand(wxString& cmd)
 	return true;
 }
 
-wxString CEditHandler::GetSystemOpenCommand(const wxString& file, bool &program_exists)
+wxString CEditHandler::GetSystemOpenCommand(wxString file, bool &program_exists)
 {
 	wxFileName fn(file);
 
@@ -986,47 +987,67 @@ wxString CEditHandler::GetSystemOpenCommand(const wxString& file, bool &program_
 	if (ext == _T(""))
 		return _T("");
 
-	wxFileType* pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
-	if (!pType)
-		return _T("");
-
-	wxString cmd;
-	if (!pType->GetOpenCommand(&cmd, wxFileType::MessageParameters(file)))
+	while (true)
 	{
+		wxFileType* pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
+		if (!pType)
+			return _T("");
+
+		wxString cmd;
+		if (!pType->GetOpenCommand(&cmd, wxFileType::MessageParameters(file)))
+		{
+			delete pType;
+			return _T("");
+		}
 		delete pType;
-		return _T("");
-	}
-	delete pType;
 
-	if (cmd.empty())
-		return wxEmptyString;
+		if (cmd.empty())
+			return wxEmptyString;
 
-	program_exists = false;
+		program_exists = false;
 
-	wxString editor;
-	if (cmd.Left(7) == _T("WX_DDE#"))
-	{
-		// See wxWidget's wxExecute in src/msw/utilsexc.cpp
-		// WX_DDE#<command>#DDE_SERVER#DDE_TOPIC#DDE_COMMAND
-		editor = cmd.Mid(7);
-		int pos = editor.Find('#');
-		if (pos < 1)
+		wxString editor;
+		if (cmd.Left(7) == _T("WX_DDE#"))
+		{
+			// See wxWidget's wxExecute in src/msw/utilsexc.cpp
+			// WX_DDE#<command>#DDE_SERVER#DDE_TOPIC#DDE_COMMAND
+			editor = cmd.Mid(7);
+			int pos = editor.Find('#');
+			if (pos < 1)
+				return cmd;
+			editor = editor.Left(pos);
+		}
+		else
+			editor = cmd;
+
+		wxString args;
+		if (!UnquoteCommand(editor, args) || editor.empty())
 			return cmd;
-		editor = editor.Left(pos);
+
+		if (!PathExpand(editor))
+			return cmd;
+
+		if (ProgramExists(editor))
+			program_exists = true;
+
+#ifdef __WXGTK__
+		int pos = args.Find(file);
+		if (pos != -1 && file.Find(' ') != -1 && file[0] != '\'' && file[0] != '"')
+		{
+			// Might need to quote filename, wxWidgets doesn't do it
+			if ((!pos || (args[pos - 1] != '\'' && args[pos - 1] != '"')) &&
+				args[pos + file.Length()] != '\'' && args[pos + file.Length()] != '"')
+			{
+				// Filename in command arguments isn't quoted. Repeat with quoted filename
+				file = _T("\"") + file + _T("\"");
+				continue;
+			}
+		}
+#endif
+		return cmd;
 	}
-	else
-		editor = cmd;
 
-	wxString args;
-	if (!UnquoteCommand(editor, args) || editor.empty())
-		return cmd;
-
-	if (!PathExpand(editor))
-		return cmd;
-
-	if (ProgramExists(editor))
-		program_exists = true;
-	return cmd;
+	return wxEmptyString;
 }
 
 wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_exists)
@@ -1199,6 +1220,16 @@ END_EVENT_TABLE()
 CEditHandlerStatusDialog::CEditHandlerStatusDialog(wxWindow* parent)
 	: m_pParent(parent)
 {
+	m_pWindowStateManager = 0;
+}
+
+CEditHandlerStatusDialog::~CEditHandlerStatusDialog()
+{
+	if (m_pWindowStateManager)
+	{
+		m_pWindowStateManager->Remember(OPTION_EDITSTATUSDIALOG_SIZE);
+		delete m_pWindowStateManager;
+	}
 }
 
 int CEditHandlerStatusDialog::ShowModal()
@@ -1295,6 +1326,9 @@ int CEditHandlerStatusDialog::ShowModal()
 	pListCtrl->SetMinSize(wxSize(pListCtrl->GetColumnWidth(0) + pListCtrl->GetColumnWidth(1) + pListCtrl->GetColumnWidth(2) + pListCtrl->GetColumnWidth(3) + 10, pListCtrl->GetMinSize().GetHeight()));
 	GetSizer()->Fit(this);
 
+	m_pWindowStateManager = new CWindowStateManager(this);
+	m_pWindowStateManager->Restore(OPTION_EDITSTATUSDIALOG_SIZE, GetSize());
+
 	SetCtrlState();
 
 	int res = wxDialogEx::ShowModal();
@@ -1595,7 +1629,14 @@ bool CNewAssociationDialog::Show(const wxString &file)
 
 void CNewAssociationDialog::SetCtrlState()
 {
-	const bool custom = XRCCTRL(*this, "ID_USE_CUSTOM", wxRadioButton)->GetValue();
+	wxRadioButton* pCustom = wxDynamicCast(FindWindow(XRCID("ID_USE_CUSTOM")), wxRadioButton);
+	if (!pCustom)
+	{
+		// Return since it can get called before dialog got fully loaded
+		return;
+	}
+
+	const bool custom = pCustom->GetValue();
 
 	XRCCTRL(*this, "ID_CUSTOM", wxTextCtrl)->Enable(custom);
 	XRCCTRL(*this, "ID_BROWSE", wxButton)->Enable(custom);
