@@ -406,7 +406,10 @@ void wxListCtrlEx::LoadColumnSettings(int widthsOptionId, int visibilityOptionId
 		if (visibleColumns.Len() >= m_columnInfo.size())
 		{
 			for (unsigned int i = 0; i < m_columnInfo.size(); i++)
-				m_columnInfo[i].shown = visibleColumns[i] == '1';
+			{
+				if (!m_columnInfo[i].fixed)
+					m_columnInfo[i].shown = visibleColumns[i] == '1';
+			}
 		}
 	}
 
@@ -434,8 +437,22 @@ void wxListCtrlEx::LoadColumnSettings(int widthsOptionId, int visibilityOptionId
 			}
 			if (i == count)
 			{
-				for (unsigned int i = 0; i < m_columnInfo.size(); i++)
-					m_columnInfo[i].order = order[i];
+				bool valid = true;
+				for (unsigned int j = 0; j < m_columnInfo.size(); j++)
+				{
+					if (!m_columnInfo[j].fixed)
+						continue;
+
+					if (j != order[j])
+					{
+						valid = false;
+						break;
+					}
+				}
+
+				if (valid)
+					for (unsigned int i = 0; i < m_columnInfo.size(); i++)
+						m_columnInfo[i].order = order[i];
 			}
 
 			delete [] order;
@@ -541,7 +558,7 @@ void wxListCtrlEx::SaveColumnWidths(unsigned int optionId)
 }
 
 
-void wxListCtrlEx::AddColumn(const wxString& name, int align, int initialWidth)
+void wxListCtrlEx::AddColumn(const wxString& name, int align, int initialWidth, bool fixed /*=false*/)
 {
 	wxASSERT(!GetColumnCount());
 
@@ -551,6 +568,7 @@ void wxListCtrlEx::AddColumn(const wxString& name, int align, int initialWidth)
 	info.width = initialWidth;
 	info.shown = true;
 	info.order = m_columnInfo.size();
+	info.fixed = fixed;
 
 	m_columnInfo.push_back(info);
 }
@@ -574,9 +592,7 @@ void wxListCtrlEx::MoveColumn(unsigned int col, unsigned int before)
 			info.order++;
 	}
 
-#ifdef __WXMSW__
 	int icon = -1;
-#endif
 
 	t_columnInfo& info = m_columnInfo[col];
 	if (info.shown)
@@ -591,9 +607,7 @@ void wxListCtrlEx::MoveColumn(unsigned int col, unsigned int before)
 				m_pVisibleColumnMapping[j - 1] = m_pVisibleColumnMapping[j];
 			info.width = GetColumnWidth(i);
 
-#ifdef __WXMSW__
 			icon = GetHeaderIconIndex(i);
-#endif
 			DeleteColumn(i);
 
 			break;
@@ -615,9 +629,7 @@ void wxListCtrlEx::MoveColumn(unsigned int col, unsigned int before)
 
 		InsertColumn(pos, info.name, info.align, info.width);
 
-#ifdef __WXMSW__
 		SetHeaderIconIndex(pos, icon);
-#endif
 	}
 	m_columnInfo[col].order = before;
 }
@@ -789,12 +801,12 @@ int wxListCtrlEx::GetColumnVisibleIndex(int col)
 	return -1;
 }
 
-#ifdef __WXMSW__
 int wxListCtrlEx::GetHeaderIconIndex(int col)
 {
 	if (col < 0 || col >= GetColumnCount())
 		return -1;
 
+#ifdef __WXMSW__
 	HWND hWnd = (HWND)GetHandle();
 	HWND header = (HWND)SendMessage(hWnd, LVM_GETHEADER, 0, 0);
 
@@ -806,6 +818,13 @@ int wxListCtrlEx::GetHeaderIconIndex(int col)
 		return -1;
 
 	return item.iImage;
+#else
+	wxListItem item;
+	if (!GetColumn(col, item))
+		return -1;
+
+	return item.GetImage();
+#endif
 }
 
 void wxListCtrlEx::SetHeaderIconIndex(int col, int icon)
@@ -813,6 +832,7 @@ void wxListCtrlEx::SetHeaderIconIndex(int col, int icon)
 	if (col < 0 || col >= GetColumnCount())
 		return;
 
+#ifdef __WXMSW__
 	HWND hWnd = (HWND)GetHandle();
 	HWND header = (HWND)SendMessage(hWnd, LVM_GETHEADER, 0, 0);
 
@@ -831,8 +851,15 @@ void wxListCtrlEx::SetHeaderIconIndex(int col, int icon)
 	else
 		item.fmt &= ~(HDF_IMAGE | HDF_BITMAP_ON_RIGHT);
 	SendMessage(header, HDM_SETITEM, col, (LPARAM)&item);
+#else
+	wxListItem item;
+	if (!GetColumn(col, item))
+		return;
+
+	item.SetImage(icon);
+	SetColumn(col, item);
+#endif
 }
-#endif //__WXMSW__
 
 void wxListCtrlEx::RefreshListOnly(bool eraseBackground /*=true*/)
 {
