@@ -18,6 +18,8 @@ public:
 
 	void EnablePrefixSearch(bool enable) { m_prefixSearch_enabled = enable; }
 
+	// Reducing item count does not reset the focused item
+ 	// if using the generic list control. Work around it.
 	void SaveSetItemCount(long count);
 
 	void ShowColumnEditor();
@@ -29,7 +31,7 @@ public:
 	void MoveColumn(unsigned int col, unsigned int before);
 	
 	// Do not call after calling LoadColumnSettings
-	void AddColumn(const wxString& name, int align, int initialWidth);
+	void AddColumn(const wxString& name, int align, int initialWidth, bool fixed = false);
 	
 	// LoadColumnSettings needs to be called exactly once after adding
 	// all columns
@@ -54,11 +56,9 @@ protected:
 	virtual wxString OnGetItemText(long item, long column) const;
 	void ResetSearchPrefix();
 
-#ifdef __WXMSW__
 	// Argument is visible column index
 	int GetHeaderIconIndex(int col);
 	void SetHeaderIconIndex(int col, int icon);
-#endif //__WXMSW__
 
 private:
 	// Keyboard prefix search
@@ -88,6 +88,7 @@ private:
 		int width;
 		bool shown;
 		unsigned int order;
+		bool fixed;
 	};
 	std::vector<t_columnInfo> m_columnInfo;
 	unsigned int *m_pVisibleColumnMapping;
