@@ -58,7 +58,7 @@ void CRecursiveOperation::StartRecursiveOperation(enum OperationMode mode, const
 	if (mode == recursive_chmod && !m_pChmodDlg)
 		return;
 
-	if ((mode == recursive_download || mode == recursive_addtoqueue || recursive_download_flatten || mode == recursive_addtoqueue_flatten) && !m_pQueue)
+	if ((mode == recursive_download || mode == recursive_addtoqueue || mode == recursive_download_flatten || mode == recursive_addtoqueue_flatten) && !m_pQueue)
 		return;
 
 	if (m_dirsToVisit.empty())
@@ -268,7 +268,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 				
 				if (m_operationMode != recursive_addtoqueue_flatten && m_operationMode != recursive_download_flatten)
 				{
-					dirToVisit.localDir += entry.name + CLocalFileSystem::path_separator;
+					dirToVisit.localDir += CQueueView::ReplaceInvalidCharacters(entry.name) + CLocalFileSystem::path_separator;
 				}
 				if (entry.link)
 				{
@@ -288,7 +288,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 			case recursive_download_flatten:
 				{
 					m_pQueue->QueueFile(m_operationMode == recursive_addtoqueue, true,
-						dir.localDir + entry.name,
+						dir.localDir + CQueueView::ReplaceInvalidCharacters(entry.name),
 						entry.name, pDirectoryListing->path, *pServer, entry.size);
 					added = true;
 				}
@@ -336,8 +336,11 @@ void CRecursiveOperation::SetChmodDialog(CChmodDialog* pChmodDialog)
 
 void CRecursiveOperation::StopRecursiveOperation()
 {
-	m_operationMode = recursive_none;
-	m_pState->NotifyHandlers(STATECHANGE_REMOTE_IDLE);
+	if (m_operationMode != recursive_none)
+	{
+		m_operationMode = recursive_none;
+		m_pState->NotifyHandlers(STATECHANGE_REMOTE_IDLE);
+	}
 	m_dirsToVisit.clear();
 	m_visitedDirs.clear();
 
