@@ -8,6 +8,7 @@
 #include "Options.h"
 #include "window_state_manager.h"
 #include "queue.h"
+#include "ipcmutex.h"
 
 class CSearchFileData : public CGenericFileData
 {
@@ -17,7 +18,7 @@ public:
 	CDirentry entry;
 };
 
-class CSearchDialogFileList : public CFileListCtrl<CSearchFileData>, public CSystemImageList
+class CSearchDialogFileList : public CFileListCtrl<CSearchFileData>
 {
 	friend class CSearchDialog;
 	friend class CSearchSortType;
@@ -38,6 +39,10 @@ protected:
 
 	void InitDateFormat();
 
+#ifdef __WXMSW__
+	virtual int GetOverlayIndex(int item);
+#endif
+
 private:
 	virtual bool CanStartComparison(wxString* pError) { return false; }
 	virtual void StartComparison() {}
@@ -61,7 +66,7 @@ class CSearchSort : public CListViewSort
 {
 public:
 	CSearchSort(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData> &fileData, enum DirSortMode dirSortMode)
-		: m_pListCtrl(pListCtrl), m_fileData(fileData), m_dirSortMode(dirSortMode)
+		: m_fileData(fileData), m_dirSortMode(dirSortMode), m_pListCtrl(pListCtrl)
 	{
 	}
 
@@ -362,7 +367,7 @@ extern wxString StripVMSRevision(const wxString& name);
 
 CSearchDialogFileList::CSearchDialogFileList(CSearchDialog* pParent, CState* pState, CQueueView* pQueue)
 	: CFileListCtrl<CSearchFileData>(pParent, pState, pQueue, true),
-	CSystemImageList(16), m_searchDialog(pParent)
+	m_searchDialog(pParent)
 {
 	m_hasParent = false;
 
@@ -374,9 +379,8 @@ CSearchDialogFileList::CSearchDialogFileList(CSearchDialog* pParent, CState* pSt
 
 	InitSort(OPTION_SEARCH_SORTORDER);
 
-#ifdef __WXMSW__
 	InitHeaderImageList();
-#endif
+
 	const unsigned long widths[7] = { 130, 130, 75, 80, 120, 80, 80 };
 
 	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0]);
@@ -582,13 +586,10 @@ bool CSearchDialog::Load()
 	m_pWindowStateManager->Restore(OPTION_SEARCH_SIZE, wxSize(750, 500));
 
 	Layout();
-	CFilter filter;
-	CFilterCondition cond;
-	cond.condition = 0;
-	cond.type = filter_name;
-	filter.filters.push_back(cond);
-	EditFilter(filter);
-	XRCCTRL(*this, "ID_CASE", wxCheckBox)->SetValue(filter.matchCase);
+
+	LoadConditions();
+	EditFilter(m_search_filter);
+	XRCCTRL(*this, "ID_CASE", wxCheckBox)->SetValue(m_search_filter.matchCase);
 
 	return true;
 }
@@ -605,6 +606,8 @@ void CSearchDialog::Run()
 
 	ShowModal();
 
+	SaveConditions();
+
 	m_pState->UnregisterHandler(this, STATECHANGE_REMOTE_IDLE);
 	m_pState->UnregisterHandler(this, STATECHANGE_REMOTE_DIR);
 	m_pState->UnblockHandlers(STATECHANGE_REMOTE_DIR);
@@ -1085,3 +1088,61 @@ void CSearchDialog::OnCharHook(wxKeyEvent& event)
 
 	event.Skip();
 }
+
+#ifdef __WXMSW__
+int CSearchDialogFileList::GetOverlayIndex(int item)
+{
+	if (item < 0 || item >= (int)m_indexMapping.size())
+		return -1;
+	int index = m_indexMapping[item];
+
+	if (m_fileData[index].entry.link)
+		return GetLinkOverlayIndex();
+
+	return 0;
+}
+#endif
+
+void CSearchDialog::LoadConditions()
+{
+	CInterProcessMutex mutex(MUTEX_SEARCHCONDITIONS);
+
+	CXmlFile file;
+	
+	TiXmlElement* pDocument = file.Load(_T("search"));
+	if (!pDocument)
+	{
+		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+		return;
+	}
+
+	TiXmlElement* pFilter = pDocument->FirstChildElement("Filter");
+	if (!pFilter)
+		return;
+
+	if (!CFilterManager::LoadFilter(pFilter, m_search_filter))
+		m_search_filter = CFilter();
+}
+
+void CSearchDialog::SaveConditions()
+{
+	CInterProcessMutex mutex(MUTEX_SEARCHCONDITIONS);
+
+	CXmlFile file;
+	
+	TiXmlElement* pDocument = file.Load(_T("search"));
+	if (!pDocument)
+	{
+		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+		return;
+	}
+
+	TiXmlElement* pFilter;
+	while ((pFilter = pDocument->FirstChildElement("Filter")))
+		pDocument->RemoveChild(pFilter);
+	pFilter = pDocument->LinkEndChild(new TiXmlElement("Filter"))->ToElement();
+
+	CFilterDialog::SaveFilter(pFilter, m_search_filter);
+
+	file.Save();
+}
