@@ -304,10 +304,10 @@ CSiteManager::~CSiteManager()
 	}
 }
 
-bool CSiteManager::Create(wxWindow* parent, const wxString& connected_site_path, const CServer* pServer /*=0*/)
+bool CSiteManager::Create(wxWindow* parent, std::vector<_connected_site> *connected_sites, const CServer* pServer /*=0*/)
 {
 	m_pSiteManagerMutex = new CInterProcessMutex(MUTEX_SITEMANAGERGLOBAL, false);
-	if (!m_pSiteManagerMutex->TryLock())
+	if (m_pSiteManagerMutex->TryLock() == 0)
 	{
 		int answer = wxMessageBox(_("The Site Manager is opened in another instance of FileZilla 3.\nDo you want to continue? Any changes made in the Site Manager won't be saved then."),
 								  _("Site Manager already open"), wxYES_NO | wxICON_QUESTION);
@@ -414,23 +414,33 @@ bool CSiteManager::Create(wxWindow* parent, const wxString& connected_site_path,
 		XRCCTRL(*this, "wxID_OK", wxButton)->SetFocus();
 #endif
 
+	m_connected_sites = connected_sites;
+	MarkConnectedSites();
+
 	if (pServer)
 		CopyAddServer(*pServer);
-	else
-		MarkConnectedSite(connected_site_path);
 
 	return true;
 }
 
-void CSiteManager::MarkConnectedSite(wxString connected_site_path)
+void CSiteManager::MarkConnectedSites()
 {
+	for (int i = 0; i < (int)m_connected_sites->size(); i++)
+		MarkConnectedSite(i);
+}
+
+void CSiteManager::MarkConnectedSite(int connected_site)
+{
+	wxString connected_site_path = (*m_connected_sites)[connected_site].old_path;
+
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
 		return;
 
 	if (connected_site_path.Left(1) == _T("1"))
 	{
-		m_changed_bookmark_path = connected_site_path;
+		// Default sites never change
+		(*m_connected_sites)[connected_site].new_path = (*m_connected_sites)[connected_site].old_path;
 		return;
 	}
 
@@ -465,7 +475,8 @@ void CSiteManager::MarkConnectedSite(wxString connected_site_path)
 		return;
 
 	CSiteManagerItemData_Site *site_data = reinterpret_cast<CSiteManagerItemData_Site* >(data);
-	site_data->is_connected_item = true;
+	wxASSERT(site_data->connected_item == -1);
+	site_data->connected_item = connected_site;
 }
 
 void CSiteManager::CreateControls(wxWindow* parent)
@@ -557,12 +568,13 @@ public:
 class CSiteManagerXmlHandler_Tree : public CSiteManagerXmlHandler
 {
 public:
-	CSiteManagerXmlHandler_Tree(wxTreeCtrl* pTree, wxTreeItemId root, const wxString& lastSelection)
-		: m_pTree(pTree), m_item(root)
+	CSiteManagerXmlHandler_Tree(wxTreeCtrl* pTree, wxTreeItemId root, const wxString& lastSelection, bool predefined)
+		: m_pTree(pTree), m_item(root), m_predefined(predefined)
 	{
 		if (!CSiteManager::UnescapeSitePath(lastSelection, m_lastSelection))
 			m_lastSelection.clear();
 		m_wrong_sel_depth = 0;
+		m_kiosk = COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE);
 	}
 
 	virtual ~CSiteManagerXmlHandler_Tree()
@@ -600,6 +612,14 @@ public:
 
 	virtual bool AddSite(CSiteManagerItemData_Site* data)
 	{
+		if (m_kiosk && !m_predefined &&
+			data->m_server.GetLogonType() == NORMAL)
+		{
+			// Clear saved password
+			data->m_server.SetLogonType(ASK);
+			data->m_server.SetUser(data->m_server.GetUser());
+		}
+
 		const wxString name(data->m_server.GetName());
 
 		wxTreeItemId newItem = m_pTree->AppendItem(m_item, name, 2, 2, data);
@@ -672,6 +692,9 @@ protected:
 	int m_wrong_sel_depth;
 
 	std::list<bool> m_expand;
+
+	bool m_predefined;
+	int m_kiosk;
 };
 
 bool CSiteManager::Load()
@@ -721,7 +744,7 @@ bool CSiteManager::Load()
 	}
 	else
 		lastSelection = _T("");
-	CSiteManagerXmlHandler_Tree handler(pTree, treeId, lastSelection);
+	CSiteManagerXmlHandler_Tree handler(pTree, treeId, lastSelection, false);
 
 	bool res = Load(pElement, &handler);
 
@@ -866,7 +889,7 @@ bool CSiteManager::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId /*=wx
 			pDocument->RemoveChild(pServers);
 			pServers = pDocument->FirstChildElement("Servers");
 		}
-		pElement = pDocument->InsertEndChild(TiXmlElement("Servers"))->ToElement();
+		pElement = pDocument->LinkEndChild(new TiXmlElement("Servers"))->ToElement();
 
 		if (!pElement)
 			return true;
@@ -910,18 +933,18 @@ bool CSiteManager::SaveChild(TiXmlElement *pElement, wxTreeItemId child)
 	CSiteManagerItemData* data = reinterpret_cast<CSiteManagerItemData* >(pTree->GetItemData(child));
 	if (!data)
 	{
-		TiXmlNode* pNode = pElement->InsertEndChild(TiXmlElement("Folder"));
+		TiXmlNode* pNode = pElement->LinkEndChild(new TiXmlElement("Folder"));
 		const bool expanded = pTree->IsExpanded(child);
 		SetTextAttribute(pNode->ToElement(), "expanded", expanded ? _T("1") : _T("0"));
 
-		pNode->InsertEndChild(TiXmlText(utf8));
+		pNode->LinkEndChild(new TiXmlText(utf8));
 
 		Save(pNode->ToElement(), child);
 	}
 	else if (data->m_type == CSiteManagerItemData::SITE)
 	{
 		CSiteManagerItemData_Site *site_data = reinterpret_cast<CSiteManagerItemData_Site* >(data);
-		TiXmlElement* pNode = pElement->InsertEndChild(TiXmlElement("Server"))->ToElement();
+		TiXmlElement* pNode = pElement->LinkEndChild(new TiXmlElement("Server"))->ToElement();
 		SetServer(pNode, site_data->m_server);
 
 		// Save comments
@@ -935,19 +958,22 @@ bool CSiteManager::SaveChild(TiXmlElement *pElement, wxTreeItemId child)
 
 		AddTextElementRaw(pNode, "SyncBrowsing", data->m_sync ? "1" : "0");
 
-		pNode->InsertEndChild(TiXmlText(utf8));
+		pNode->LinkEndChild(new TiXmlText(utf8));
 
 		Save(pNode, child);
 
-		if (site_data->is_connected_item)
+		if (site_data->connected_item != -1)
 		{
-			m_changed_bookmark_path = GetSitePath(child);
-			m_changed_bookmark_server = site_data->m_server;
+			if ((*m_connected_sites)[site_data->connected_item].server == site_data->m_server)
+			{
+				(*m_connected_sites)[site_data->connected_item].new_path = GetSitePath(child);
+				(*m_connected_sites)[site_data->connected_item].server = site_data->m_server;
+			}
 		}
 	}
 	else
 	{
-		TiXmlElement* pNode = pElement->InsertEndChild(TiXmlElement("Bookmark"))->ToElement();
+		TiXmlElement* pNode = pElement->LinkEndChild(new TiXmlElement("Bookmark"))->ToElement();
 
 		AddTextElement(pNode, "Name", name);
 
@@ -1028,6 +1054,7 @@ bool CSiteManager::Verify()
 		}
 
 		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) != 0 &&
+			!IsPredefinedItem(item) &&
 			(logon_type == ACCOUNT || logon_type == NORMAL))
 		{
 			XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetFocus();
@@ -1504,10 +1531,10 @@ bool CSiteManager::GetServer(CSiteManagerItemData_Site& data)
 			data.m_localDir = pData->m_localDir;
 		if (!pData->m_remoteDir.IsEmpty())
 			data.m_remoteDir = pData->m_remoteDir;
-		if (!data.m_localDir.empty() && !data.m_remoteDir.IsEmpty())
-			data.m_sync = true;
-		else
+		if (data.m_localDir.empty() || data.m_remoteDir.IsEmpty())
 			data.m_sync = false;
+		else
+			data.m_sync = pData->m_sync;
 	}
 	else
 		data = *(CSiteManagerItemData_Site *)pData;
@@ -1836,7 +1863,7 @@ void CSiteManager::OnCopySite(wxCommandEvent& event)
 	if (data->m_type == CSiteManagerItemData::SITE)
 	{
 		CSiteManagerItemData_Site* newData = new CSiteManagerItemData_Site(*(CSiteManagerItemData_Site *)data);
-		newData->is_connected_item = false;
+		newData->connected_item = -1;
 		newItem = pTree->AppendItem(parent, newName, 2, 2, newData);
 
 		wxTreeItemIdValue cookie;
@@ -1898,7 +1925,7 @@ bool CSiteManager::LoadDefaultSites()
 	}
 	else
 		lastSelection = _T("");
-	CSiteManagerXmlHandler_Tree handler(pTree, m_predefinedSites, lastSelection);
+	CSiteManagerXmlHandler_Tree handler(pTree, m_predefinedSites, lastSelection, true);
 
 	Load(pElement, &handler);
 
@@ -2311,7 +2338,7 @@ bool CSiteManager::MoveItems(wxTreeItemId source, wxTreeItemId target, bool copy
 		else if (data->m_type == CSiteManagerItemData::SITE)
 		{
 			CSiteManagerItemData_Site* newData = new CSiteManagerItemData_Site(*(CSiteManagerItemData_Site *)data);
-			newData->is_connected_item = false;
+			newData->connected_item = -1;
 			pTree->SetItemData(newItem, newData);
 		}
 		else
@@ -2420,7 +2447,7 @@ void CSiteManager::AddNewSite(wxTreeItemId parent, const CServer& server, bool c
 
 	CSiteManagerItemData_Site* pData = new CSiteManagerItemData_Site(server);
 	if (connected)
-		pData->is_connected_item = true;
+		pData->connected_item = 0;
 
 	wxTreeItemId newItem = pTree->AppendItem(parent, name, 2, 2, pData);
 	pTree->SortChildren(parent);
@@ -2656,7 +2683,7 @@ void CSiteManager::OnExportSelected(wxCommandEvent& event)
 
 	TiXmlElement* exportRoot = xml.CreateEmpty();
 
-	TiXmlElement* pServers = exportRoot->InsertEndChild(TiXmlElement("Servers"))->ToElement();
+	TiXmlElement* pServers = exportRoot->LinkEndChild(new TiXmlElement("Servers"))->ToElement();
 	SaveChild(pServers, m_contextMenuItem);
 
 	wxString error;
@@ -2881,13 +2908,13 @@ wxString CSiteManager::AddServer(CServer server)
 
 	server.SetName(name);
 
-	TiXmlElement* pServer = pElement->InsertEndChild(TiXmlElement("Server"))->ToElement();
+	TiXmlElement* pServer = pElement->LinkEndChild(new TiXmlElement("Server"))->ToElement();
 	SetServer(pServer, server);
 
 	char* utf8 = ConvUTF8(name);
 	if (utf8)
 	{
-		pServer->InsertEndChild(TiXmlText(utf8));
+		pServer->LinkEndChild(new TiXmlText(utf8));
 		delete [] utf8;
 	}
 
@@ -3009,7 +3036,7 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	if (pInsertBefore)
 		pBookmark = pChild->InsertBeforeChild(pInsertBefore, TiXmlElement("Bookmark"))->ToElement();
 	else
-		pBookmark = pChild->InsertEndChild(TiXmlElement("Bookmark"))->ToElement();
+		pBookmark = pChild->LinkEndChild(new TiXmlElement("Bookmark"))->ToElement();
 	AddTextElement(pBookmark, "Name", name);
 	if (!local_dir.empty())
 		AddTextElement(pBookmark, "LocalDir", local_dir);
@@ -3092,11 +3119,3 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 
 	return true;
 }
-
-wxString CSiteManager::GetChangedBookmarkPath(const CServer* pServer)
-{
-	if (pServer && m_changed_bookmark_server != *pServer)
-		return wxEmptyString;
-	
-	return m_changed_bookmark_path;
-}
