@@ -11,11 +11,16 @@
 #include "local_filesys.h"
 #include "listingcomparison.h"
 
-CContextManager the_context_manager;
+CContextManager CContextManager::m_the_context_manager;
+
+CContextManager::CContextManager()
+{
+	m_current_context = -1;
+}
 
 CContextManager* CContextManager::Get()
 {
-	return &the_context_manager;
+	return &m_the_context_manager;
 }
 
 CState* CContextManager::CreateState(CMainFrame* pMainFrame)
@@ -26,6 +31,8 @@ CState* CContextManager::CreateState(CMainFrame* pMainFrame)
 
 	m_contexts.push_back(pState);
 
+	NotifyHandlers(pState, STATECHANGE_NEWCONTEXT, _T(""), 0, false);
+
 	return pState;
 }
 
@@ -41,16 +48,48 @@ void CContextManager::DestroyState(CState* pState)
 			m_current_context--;
 		else if ((int)i == m_current_context)
 		{
-			m_current_context--;
-			// TODO context change
+			if (i >= m_contexts.size())
+				m_current_context--;
+			NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
 		}
 
 		break;
 	}
 
+	NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0, false);
 	delete pState;
 }
 
+void CContextManager::SetCurrentContext(CState* pState)
+{
+	if (GetCurrentContext() == pState)
+		return;
+
+	for (unsigned int i = 0; i < m_contexts.size(); i++)
+	{
+		if (m_contexts[i] != pState)
+			continue;
+
+		m_current_context = i;
+		NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
+	}
+}
+
+void CContextManager::DestroyAllStates()
+{
+	m_current_context = -1;
+	NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
+
+	while (!m_contexts.empty())
+	{
+		CState* pState = m_contexts.back();
+		m_contexts.pop_back();
+
+		NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0, false);
+		delete pState;
+	}
+}
+
 void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool current_only, bool blockable)
 {
 	wxASSERT(pHandler);
@@ -105,6 +144,12 @@ void CContextManager::UnregisterHandler(CStateEventHandler* pHandler, enum t_sta
 	}
 }
 
+size_t CContextManager::HandlerCount(t_statechange_notifications notification) const
+{
+	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
+	return m_handlers[notification].size();
+}
+
 void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2, bool blocked)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
@@ -115,10 +160,34 @@ void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications
 		if (blocked && iter->blockable)
 			continue;
 
+		if (iter->current_only && pState != GetCurrentContext())
+			continue;
+
 		iter->pHandler->OnStateChange(pState, notification, data, data2);
 	}
 }
 
+CState* CContextManager::GetCurrentContext()
+{
+	if (m_current_context == -1)
+		return 0;
+
+	return m_contexts[m_current_context];
+}
+
+void CContextManager::NotifyAllHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+{
+	for (unsigned int i = 0; i < m_contexts.size(); i++)
+		m_contexts[i]->NotifyHandlers(notification, data, data2);
+}
+
+void CContextManager::NotifyGlobalHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+{
+	const std::list<t_handler> &handlers = m_handlers[notification];
+	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+		iter->pHandler->OnStateChange(0, notification, data, data2);
+}
+
 CState::CState(CMainFrame* pMainFrame)
 {
 	memset(m_blocked, 0, sizeof(m_blocked));
@@ -127,6 +196,7 @@ CState::CState(CMainFrame* pMainFrame)
 
 	m_pDirectoryListing = 0;
 	m_pServer = 0;
+	m_title = _("Not connected");
 	m_successful_connect = 0;
 
 	m_pEngine = 0;
@@ -137,6 +207,8 @@ CState::CState(CMainFrame* pMainFrame)
 
 	m_sync_browse.is_changing = false;
 	m_sync_browse.compare = false;
+
+	m_localDir.SetPath(CLocalPath::path_separator);
 }
 
 CState::~CState()
@@ -266,7 +338,7 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 		}
 	}
 	else
-		COptions::Get()->SetOption(OPTION_LASTSERVERPATH, pDirectoryListing->path.GetSafePath());
+		m_last_path = pDirectoryListing->path;
 
 	if (m_pDirectoryListing && m_pDirectoryListing->path == pDirectoryListing->path &&
         pDirectoryListing->m_failed)
@@ -283,7 +355,7 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 	else
 		NotifyHandlers(STATECHANGE_REMOTE_DIR_MODIFIED);
 
-	if (m_sync_browse.is_changing)
+	if (m_sync_browse.is_changing && !modified)
 	{
 		m_sync_browse.is_changing = false;
 		if (m_pDirectoryListing->path != m_sync_browse.remote_root && !m_pDirectoryListing->path.IsSubdirOf(m_sync_browse.remote_root, false))
@@ -391,7 +463,9 @@ void CState::SetServer(const CServer* server)
 {
 	if (m_pServer)
 	{
-		if (server && *server == *m_pServer)
+		if (server && *server == *m_pServer &&
+			server->GetName() == m_pServer->GetName() &&
+			server->MaximumMultipleConnections() == m_pServer->MaximumMultipleConnections())
 		{
 			// Nothing changes
 			return;
@@ -401,9 +475,24 @@ void CState::SetServer(const CServer* server)
 		delete m_pServer;
 	}
 	if (server)
+	{
+		if (m_last_server != *server)
+			m_last_path.Clear();
+		m_last_server = *server;
+
 		m_pServer = new CServer(*server);
+
+		const wxString& name = server->GetName();
+		if (!name.IsEmpty())
+			m_title = name + _T(" - ") + server->FormatServer();
+		else
+			m_title = server->FormatServer();
+	}
 	else
+	{
 		m_pServer = 0;
+		m_title = _("Not connected");
+	}
 
 	m_successful_connect = false;
 
@@ -415,29 +504,41 @@ const CServer* CState::GetServer() const
 	return m_pServer;
 }
 
-bool CState::Connect(const CServer& server, bool askBreak, const CServerPath& path /*=CServerPath()*/)
+wxString CState::GetTitle() const
+{
+	return m_title;
+}
+
+bool CState::Connect(const CServer& server, const CServerPath& path /*=CServerPath()*/)
 {
 	if (!m_pEngine)
 		return false;
 	if (m_pEngine->IsConnected() || m_pEngine->IsBusy() || !m_pCommandQueue->Idle())
-	{
-		if (askBreak)
-			if (wxMessageBox(_("Break current connection?"), _T("FileZilla"), wxYES_NO | wxICON_QUESTION) != wxYES)
-				return false;
 		m_pCommandQueue->Cancel();
-	}
+	m_pRecursiveOperation->StopRecursiveOperation();
+	SetSyncBrowse(false);
 
 	m_pCommandQueue->ProcessCommand(new CConnectCommand(server));
 	m_pCommandQueue->ProcessCommand(new CListCommand(path, _T(""), LIST_FLAG_FALLBACK_CURRENT));
 
-	COptions::Get()->SetLastServer(server);
-	COptions::Get()->SetOption(OPTION_LASTSERVERPATH, path.GetSafePath());
+	SetServer(&server);
 
-	const wxString& name = server.GetName();
-	if (!name.IsEmpty())
-		m_pMainFrame->SetTitle(name + _T(" - ") + server.FormatServer() + _T(" - FileZilla"));
-	else
-		m_pMainFrame->SetTitle(server.FormatServer() + _T(" - FileZilla"));
+	return true;
+}
+
+bool CState::Disconnect()
+{
+	if (!m_pEngine)
+		return false;
+
+	if (!IsRemoteConnected())
+		return true;
+	
+	if (!IsRemoteIdle())
+		return false;
+
+	SetServer(0);
+	m_pCommandQueue->ProcessCommand(new CDisconnectCommand());
 
 	return true;
 }
@@ -451,7 +552,7 @@ bool CState::CreateEngine()
 	m_pEngine = new CFileZillaEngine();
 	m_pEngine->Init(m_pMainFrame, COptions::Get());
 
-	m_pCommandQueue = new CCommandQueue(m_pEngine, m_pMainFrame);
+	m_pCommandQueue = new CCommandQueue(m_pEngine, m_pMainFrame, this);
 
 	return true;
 }
@@ -468,6 +569,8 @@ void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_no
 {
 	wxASSERT(pHandler);
 	wxASSERT(pHandler->m_pState == this);
+	if (pHandler->m_pState != this)
+		return;
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
 
 	std::list<t_handler> &handlers = m_handlers[notification];
@@ -562,11 +665,8 @@ void CState::NotifyHandlers(enum t_statechange_notifications notification, const
 }
 
 CStateEventHandler::CStateEventHandler(CState* pState)
+	: m_pState(pState)
 {
-	if (!pState)
-		return;
-
-	m_pState = pState;
 }
 
 CStateEventHandler::~CStateEventHandler()
@@ -783,7 +883,7 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 		if (!iter->dir)
 			continue;
 
-		m_pRecursiveOperation->AddDirectoryToVisit(pRemoteDataObject->GetServerPath(), iter->name, path.GetPath() + iter->name, iter->link);
+		m_pRecursiveOperation->AddDirectoryToVisit(pRemoteDataObject->GetServerPath(), iter->name, path.GetPath() + CQueueView::ReplaceInvalidCharacters(iter->name), iter->link);
 	}
 
 	if (m_pComparisonManager->IsComparing())
@@ -1030,3 +1130,14 @@ CServerPath CState::GetSynchronizedDirectory(CLocalPath local_path)
 
 	return remote_path;
 }
+
+bool CState::RefreshRemote()
+{
+	if (!m_pCommandQueue)
+		return false;
+	
+	if (!IsRemoteConnected() || !IsRemoteIdle())
+		return false;
+
+	return ChangeRemoteDir(GetRemotePath(), _T(""), LIST_FLAG_REFRESH);
+}
