@@ -11,6 +11,7 @@
 #ifdef __WXMSW__
 #include <wx/dynlib.h> // Used by GetDownloadDir
 #endif //__WXMSW__
+#include "dialogex.h"
 
 // This is ugly but does the job
 #define SHA512_STANDALONE
@@ -18,6 +19,14 @@ typedef unsigned int uint32;
 #include "../putty/int64.h"
 #include "../putty/sshsh512.c"
 
+#if 0
+// Used inside of wx to create the next/back buttons on the wizard.
+// Listed here so that they are included in the translations
+_("&Next >");
+_("< &Back");
+_("&Finish");
+#endif
+
 #define MAXCHECKPROGRESS 11 // Maximum value of progress bar
 
 BEGIN_EVENT_TABLE(CUpdateWizard, wxWizard)
@@ -123,12 +132,15 @@ CUpdateWizard::CUpdateWizard(wxWindow* pParent)
 
 	m_statusTimer.SetOwner(this);
 	m_autoCheckTimer.SetOwner(this);
+	m_busy_timer.SetOwner(this);
 
 	m_autoUpdateCheckRunning = false;
 
 	m_loaded = false;
 	m_updateShown = false;
+	m_menuUpdated = false;
 	m_start_check = false;
+	m_successfully_downloaded = false;
 }
 
 CUpdateWizard::~CUpdateWizard()
@@ -569,6 +581,8 @@ void CUpdateWizard::OnEngineEvent(wxEvent& event)
 #endif
 					pText->Show();
 					RewrapPage(2);
+
+					m_successfully_downloaded = true;
 				}
 			}
 			break;
@@ -644,10 +658,10 @@ void CUpdateWizard::OnEngineEvent(wxEvent& event)
 
 void CUpdateWizard::ParseData()
 {
-	const wxULongLong ownVersionNumber = CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion());
+	const wxLongLong ownVersionNumber = CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion());
 
 	wxString newVersion;
-	wxULongLong newVersionNumber = 0;
+	wxLongLong newVersionNumber = -1;
 	wxString newUrl;
 	wxString newChecksum;
 
@@ -733,7 +747,8 @@ void CUpdateWizard::ParseData()
 		}
 		else
 		{
-			if (CBuildInfo::ConvertToVersionNumber(versionOrDate) <= ownVersionNumber)
+			wxLongLong v = CBuildInfo::ConvertToVersionNumber(versionOrDate);
+			if (v <= ownVersionNumber)
 				continue;
 		}
 
@@ -813,6 +828,8 @@ void CUpdateWizard::OnTimer(wxTimerEvent& event)
 			StartUpdateCheck();
 		}
 	}
+	else if (event.GetId() == m_busy_timer.GetId())
+		DisplayUpdateAvailability(true);
 }
 
 void CUpdateWizard::SetTransferStatus(const CTransferStatus* pStatus)
@@ -876,7 +893,7 @@ void CUpdateWizard::OnFinish(wxWizardEvent& event)
 #ifdef __WXMSW__
 	if (m_currentPage == 2)
 	{
-		wxExecute(m_localFile);
+		wxExecute(_T("\"") + m_localFile +  _T("\" /update"));
 		CMainFrame* pFrame = (CMainFrame*)m_parent;
 		pFrame->Close();
 	}
@@ -925,7 +942,11 @@ bool CUpdateWizard::CanAutoCheckForUpdateNow()
 		// Last check in future
 		return true;
 	
-	const int days = COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_INTERVAL) * 3600 * 24;
+	int days;
+	if (CBuildInfo::GetBuildType() == _T("official") && CBuildInfo::IsUnstable())
+		days = 3600 * 24;
+	else
+		days = COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_INTERVAL) * 3600 * 24;
 	if (span.GetSeconds() >= days)
 		return true;
 
@@ -955,11 +976,8 @@ void CUpdateWizard::StartUpdateCheck()
 		FailedTransfer();
 }
 
-void CUpdateWizard::DisplayUpdateAvailability(bool showDialog, bool forceMenu /*=false*/)
+void CUpdateWizard::DisplayUpdateAvailability(bool showDialog)
 {
-	if (m_updateShown && !forceMenu)
-		return;
-
 	COptions* pOptions = COptions::Get();
 
 	if (CBuildInfo::GetVersion() == _T("custom build"))
@@ -969,30 +987,47 @@ void CUpdateWizard::DisplayUpdateAvailability(bool showDialog, bool forceMenu /*
 	if (newVersion == _T(""))
 		return;
 
-	if (CBuildInfo::ConvertToVersionNumber(newVersion) <= CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion()))
+	wxLongLong v = CBuildInfo::ConvertToVersionNumber(newVersion);
+	if (v <= CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion()))
 	{
 		pOptions->SetOption(OPTION_UPDATECHECK_NEWVERSION, _T(""));
 		return;
 	}
 
+	if (!m_menuUpdated)
+	{
+		m_menuUpdated = true;
+
 #ifdef __WXMSW__
-	// All open menus need to be closed or app will become unresponsive.
-	::EndMenu();
+		// All open menus need to be closed or app will become unresponsive.
+		::EndMenu();
 #endif
 
-	m_updateShown = true;
+		CMainFrame* pFrame = (CMainFrame*)m_parent;
 
-	CMainFrame* pFrame = (CMainFrame*)m_parent;
-		
-	wxMenu* pMenu = new wxMenu();
-	const wxString& name = wxString::Format(_("&Version %s"), pOptions->GetOption(OPTION_UPDATECHECK_NEWVERSION).c_str());
-	pMenu->Append(XRCID("ID_CHECKFORUPDATES"), name);
-	wxMenuBar* pMenuBar = pFrame->GetMenuBar();
-	if (pMenuBar)
-		pMenuBar->Append(pMenu, _("&New version available!"));
+		wxMenu* pMenu = new wxMenu();
+		const wxString& name = wxString::Format(_("&Version %s"), pOptions->GetOption(OPTION_UPDATECHECK_NEWVERSION).c_str());
+		pMenu->Append(XRCID("ID_CHECKFORUPDATES"), name);
+		wxMenuBar* pMenuBar = pFrame->GetMenuBar();
+		if (pMenuBar)
+			pMenuBar->Append(pMenu, _("&New version available!"));
+	}
 
-	if (showDialog)
+	if (showDialog && !m_updateShown)
 	{
+		if (wxDialogEx::ShownDialogs())
+		{
+			m_busy_timer.Start(1000, true);
+			return;
+		}
+
+		m_updateShown = true;
+
+#ifdef __WXMSW__
+		// All open menus need to be closed or app will become unresponsive.
+		::EndMenu();
+#endif
+
 		CUpdateWizard dlg(m_parent);
 		if (dlg.Load())
 			dlg.Run();
@@ -1203,7 +1238,7 @@ void CUpdateWizard::OnCancel(wxWizardEvent& event)
 {
 	delete m_pEngine;
 	m_pEngine = 0;
-	if (m_localFile != _T(""))
+	if (m_localFile != _T("") && !m_successfully_downloaded)
 		wxRemoveFile(m_localFile);
 }
 
