@@ -12,32 +12,18 @@ CVerifyCertDialog::~CVerifyCertDialog()
 		delete [] iter->data;
 }
 
-void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotification, bool displayOnly /*=false*/)
+bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 {
-	LoadTrustedCerts();
-
-	wxDialogEx* pDlg = new wxDialogEx;
-	if (displayOnly)
-		pDlg->Load(0, _T("ID_DISPLAYCERT"));
-	else
-	{
-		pDlg->Load(0, _T("ID_VERIFYCERT"));
-
-		pDlg->WrapText(pDlg, XRCID("ID_DESC"), 400);
-	}
-
-	pDlg->SetLabel(XRCID("ID_HOST"), wxString::Format(_T("%s:%d"), pNotification->GetHost().c_str(), pNotification->GetPort()));
-
 	bool warning = false;
-	if (pNotification->GetActivationTime().IsValid())
+	if (cert.GetActivationTime().IsValid())
 	{
-		if (pNotification->GetActivationTime() > wxDateTime::Now())
+		if (cert.GetActivationTime() > wxDateTime::Now())
 		{
-			pDlg->SetLabel(XRCID("ID_ACTIVATION_TIME"), wxString::Format(_("%s - Not yet valid!"), pNotification->GetActivationTime().FormatDate().c_str()));
+			pDlg->SetLabel(XRCID("ID_ACTIVATION_TIME"), wxString::Format(_("%s - Not yet valid!"), cert.GetActivationTime().FormatDate().c_str()));
 			warning = true;
 		}
 		else
-			pDlg->SetLabel(XRCID("ID_ACTIVATION_TIME"), pNotification->GetActivationTime().FormatDate());
+			pDlg->SetLabel(XRCID("ID_ACTIVATION_TIME"), cert.GetActivationTime().FormatDate());
 	}
 	else
 	{
@@ -45,15 +31,15 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotifi
 		pDlg->SetLabel(XRCID("ID_ACTIVATION_TIME"), _("Invalid date"));
 	}
 
-	if (pNotification->GetExpirationTime().IsValid())
+	if (cert.GetExpirationTime().IsValid())
 	{
-		if (pNotification->GetExpirationTime() < wxDateTime::Now())
+		if (cert.GetExpirationTime() < wxDateTime::Now())
 		{
-			pDlg->SetLabel(XRCID("ID_EXPIRATION_TIME"), wxString::Format(_("%s - Certificate expired!"), pNotification->GetExpirationTime().FormatDate().c_str()));
+			pDlg->SetLabel(XRCID("ID_EXPIRATION_TIME"), wxString::Format(_("%s - Certificate expired!"), cert.GetExpirationTime().FormatDate().c_str()));
 			warning = true;
 		}
 		else
-			pDlg->SetLabel(XRCID("ID_EXPIRATION_TIME"), pNotification->GetExpirationTime().FormatDate());
+			pDlg->SetLabel(XRCID("ID_EXPIRATION_TIME"), cert.GetExpirationTime().FormatDate());
 	}
 	else
 	{
@@ -61,38 +47,78 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotifi
 		pDlg->SetLabel(XRCID("ID_EXPIRATION_TIME"), _("Invalid date"));
 	}
 
-	if (pNotification->GetSerial() != _T(""))
-		pDlg->SetLabel(XRCID("ID_SERIAL"), pNotification->GetSerial());
+	if (cert.GetSerial() != _T(""))
+		pDlg->SetLabel(XRCID("ID_SERIAL"), cert.GetSerial());
 	else
 		pDlg->SetLabel(XRCID("ID_SERIAL"), _("None"));
 
-	pDlg->SetLabel(XRCID("ID_PKALGO"), wxString::Format(_("%s with %d bits"), pNotification->GetPkAlgoName().c_str(), pNotification->GetPkAlgoBits()));
+	pDlg->SetLabel(XRCID("ID_PKALGO"), wxString::Format(_("%s with %d bits"), cert.GetPkAlgoName().c_str(), cert.GetPkAlgoBits()));
 
-	pDlg->SetLabel(XRCID("ID_FINGERPRINT_MD5"), pNotification->GetFingerPrintMD5());
-	pDlg->SetLabel(XRCID("ID_FINGERPRINT_SHA1"), pNotification->GetFingerPrintSHA1());
+	pDlg->SetLabel(XRCID("ID_FINGERPRINT_MD5"), cert.GetFingerPrintMD5());
+	pDlg->SetLabel(XRCID("ID_FINGERPRINT_SHA1"), cert.GetFingerPrintSHA1());
 
-	wxSizer* pSizer = XRCCTRL(*pDlg, "ID_SUBJECT_DUMMY", wxStaticText)->GetContainingSizer();
-	ParseDN(pDlg, pNotification->GetSubject(), pSizer);
-	XRCCTRL(*pDlg, "ID_SUBJECT_DUMMY", wxStaticText)->Destroy();
+	ParseDN(pDlg, cert.GetSubject(), m_pSubjectSizer);
+	ParseDN(pDlg, cert.GetIssuer(), m_pIssuerSizer);
 
-	pSizer = XRCCTRL(*pDlg, "ID_ISSUER_DUMMY", wxStaticText)->GetContainingSizer();
-	ParseDN(pDlg, pNotification->GetIssuer(), pSizer);
-	XRCCTRL(*pDlg, "ID_ISSUER_DUMMY", wxStaticText)->Destroy();
+	return warning;
+}
 
-	pDlg->SetLabel(XRCID("ID_CIPHER"), pNotification->GetSessionCipher());
-	pDlg->SetLabel(XRCID("ID_MAC"), pNotification->GetSessionMac());
+void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotification, bool displayOnly /*=false*/)
+{
+	LoadTrustedCerts();
+
+	m_pDlg = new wxDialogEx;
+	if (displayOnly)
+		m_pDlg->Load(0, _T("ID_DISPLAYCERT"));
+	else
+	{
+		m_pDlg->Load(0, _T("ID_VERIFYCERT"));
+
+		m_pDlg->WrapText(m_pDlg, XRCID("ID_DESC"), 400);
+	}
+
+	m_certificates = pNotification->GetCertificates();
+	if (m_certificates.size() == 1)
+	{
+		XRCCTRL(*m_pDlg, "ID_CHAIN_DESC", wxStaticText)->Hide();
+		XRCCTRL(*m_pDlg, "ID_CHAIN", wxChoice)->Hide();
+	}
+	else
+	{
+		wxChoice* pChoice = XRCCTRL(*m_pDlg, "ID_CHAIN", wxChoice);
+		for (unsigned int i = 0; i < m_certificates.size(); i++)
+		{
+			pChoice->Append(wxString::Format(_T("%d"), i));
+		}
+		pChoice->SetSelection(0);
+
+		pChoice->Connect(wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler(CVerifyCertDialog::OnCertificateChoice), 0, this);
+	}
+
+	m_pDlg->SetLabel(XRCID("ID_HOST"), wxString::Format(_T("%s:%d"), pNotification->GetHost().c_str(), pNotification->GetPort()));
+
+	m_pSubjectSizer = XRCCTRL(*m_pDlg, "ID_SUBJECT_DUMMY", wxStaticText)->GetContainingSizer();
+	XRCCTRL(*m_pDlg, "ID_SUBJECT_DUMMY", wxStaticText)->Destroy();
+
+	m_pIssuerSizer = XRCCTRL(*m_pDlg, "ID_ISSUER_DUMMY", wxStaticText)->GetContainingSizer();
+	XRCCTRL(*m_pDlg, "ID_ISSUER_DUMMY", wxStaticText)->Destroy();
+
+	bool warning = DisplayCert(m_pDlg, m_certificates[0]);
+
+	m_pDlg->SetLabel(XRCID("ID_CIPHER"), pNotification->GetSessionCipher());
+	m_pDlg->SetLabel(XRCID("ID_MAC"), pNotification->GetSessionMac());
 
 	if (warning)
 	{
-		XRCCTRL(*pDlg, "ID_IMAGE", wxStaticBitmap)->SetBitmap(wxArtProvider::GetBitmap(wxART_WARNING));
+		XRCCTRL(*m_pDlg, "ID_IMAGE", wxStaticBitmap)->SetBitmap(wxArtProvider::GetBitmap(wxART_WARNING));
 		if (!displayOnly)
-			XRCCTRL(*pDlg, "ID_ALWAYS", wxCheckBox)->Enable(false);
+			XRCCTRL(*m_pDlg, "ID_ALWAYS", wxCheckBox)->Enable(false);
 	}
 
-	pDlg->GetSizer()->Fit(pDlg);
-	pDlg->GetSizer()->SetSizeHints(pDlg);
+	m_pDlg->GetSizer()->Fit(m_pDlg);
+	m_pDlg->GetSizer()->SetSizeHints(m_pDlg);
 
-	int res = pDlg->ShowModal();
+	int res = m_pDlg->ShowModal();
 
 	if (!displayOnly)
 	{
@@ -102,12 +128,12 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotifi
 
 			pNotification->m_trusted = true;
 
-			if (!warning && XRCCTRL(*pDlg, "ID_ALWAYS", wxCheckBox)->GetValue())
+			if (!warning && XRCCTRL(*m_pDlg, "ID_ALWAYS", wxCheckBox)->GetValue())
 				SetPermanentlyTrusted(pNotification);
 			else
 			{
 				t_certData cert;
-				const unsigned char* data = pNotification->GetRawData(cert.len);
+				const unsigned char* data = m_certificates[0].GetRawData(cert.len);
 				cert.data = new unsigned char[cert.len];
 				memcpy(cert.data, data, cert.len);
 				m_sessionTrustedCerts.push_back(cert);
@@ -117,11 +143,14 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotifi
 			pNotification->m_trusted = false;
 	}
 
-	delete pDlg;
+	delete m_pDlg;
+	m_pDlg = 0;
 }
 
 void CVerifyCertDialog::ParseDN(wxDialog* pDlg, const wxString& dn, wxSizer* pSizer)
 {
+	pSizer->Clear(true);
+
 	wxStringTokenizer tokens(dn, _T(","));
 
 	std::list<wxString> tokenlist;
@@ -130,13 +159,18 @@ void CVerifyCertDialog::ParseDN(wxDialog* pDlg, const wxString& dn, wxSizer* pSi
 
 	ParseDN_by_prefix(pDlg, tokenlist, _T("CN"), _("Common name:"), pSizer);
 	ParseDN_by_prefix(pDlg, tokenlist, _T("O"), _("Organization:"), pSizer);
+	ParseDN_by_prefix(pDlg, tokenlist, _T("2.5.4.15"), _("Business category:"), pSizer, true);
 	ParseDN_by_prefix(pDlg, tokenlist, _T("OU"), _("Unit:"), pSizer);
 	ParseDN_by_prefix(pDlg, tokenlist, _T("T"), _("Title:"), pSizer);
 	ParseDN_by_prefix(pDlg, tokenlist, _T("C"), _("Country:"), pSizer);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("ST"), _("State:"), pSizer);
+	ParseDN_by_prefix(pDlg, tokenlist, _T("ST"), _("State or province:"), pSizer);
 	ParseDN_by_prefix(pDlg, tokenlist, _T("L"), _("Locality:"), pSizer);
+	ParseDN_by_prefix(pDlg, tokenlist, _T("2.5.4.17"), _("Postal code:"), pSizer, true);
 	ParseDN_by_prefix(pDlg, tokenlist, _T("STREET"), _("Street:"), pSizer);
 	ParseDN_by_prefix(pDlg, tokenlist, _T("EMAIL"), _("E-Mail:"), pSizer);
+	ParseDN_by_prefix(pDlg, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.3"), _("Jurisdiction country:"), pSizer, true);
+	ParseDN_by_prefix(pDlg, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.2"), _("Jurisdiction state or province:"), pSizer, true);	
+	ParseDN_by_prefix(pDlg, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.1"), _("Jurisdiction locality:"), pSizer, true);
 
 	if (!tokenlist.empty())
 	{
@@ -149,7 +183,7 @@ void CVerifyCertDialog::ParseDN(wxDialog* pDlg, const wxString& dn, wxSizer* pSi
 	}
 }
 
-void CVerifyCertDialog::ParseDN_by_prefix(wxDialog* pDlg, std::list<wxString>& tokens, wxString prefix, const wxString& name, wxSizer* pSizer)
+void CVerifyCertDialog::ParseDN_by_prefix(wxDialog* pDlg, std::list<wxString>& tokens, wxString prefix, const wxString& name, wxSizer* pSizer, bool decode /*=false*/)
 {
 	prefix += _T("=");
 	const int len = prefix.Length();
@@ -190,6 +224,9 @@ void CVerifyCertDialog::ParseDN_by_prefix(wxDialog* pDlg, std::list<wxString>& t
 		tokens.erase(remove);
 	}
 
+	if (decode)
+		value = DecodeValue(value);
+
 	if (value != _T(""))
 	{
 		pSizer->Add(new wxStaticText(pDlg, wxID_ANY, name));
@@ -204,7 +241,8 @@ bool CVerifyCertDialog::IsTrusted(CCertificateNotification* pNotification)
 	wxASSERT(pNotification);
 
 	unsigned int len;
-	const unsigned char* data = pNotification->GetRawData(len);
+	CCertificate cert =  pNotification->GetCertificates()[0];
+	const unsigned char* data = cert.GetRawData(len);
 
 	return IsTrusted(data, len, false);
 }
@@ -364,8 +402,9 @@ void CVerifyCertDialog::LoadTrustedCerts(bool close /*=true*/)
 
 void CVerifyCertDialog::SetPermanentlyTrusted(const CCertificateNotification* const pNotification)
 {
+	const CCertificate certificate = pNotification->GetCertificates()[0];
 	unsigned int len;
-	const unsigned char* const data = pNotification->GetRawData(len);
+	const unsigned char* const data = certificate.GetRawData(len);
 
 	CReentrantInterProcessMutexLocker mutex(MUTEX_TRUSTEDCERTS);
 	LoadTrustedCerts(false);
@@ -397,19 +436,78 @@ void CVerifyCertDialog::SetPermanentlyTrusted(const CCertificateNotification* co
 
 	TiXmlElement* pCerts = pElement->FirstChildElement("TrustedCerts");
 	if (!pCerts)
-		pCerts = pElement->InsertEndChild(TiXmlElement("TrustedCerts"))->ToElement();
+		pCerts = pElement->LinkEndChild(new TiXmlElement("TrustedCerts"))->ToElement();
 
-	TiXmlElement* pCert = pCerts->InsertEndChild(TiXmlElement("Certificate"))->ToElement();
+	TiXmlElement* pCert = pCerts->LinkEndChild(new TiXmlElement("Certificate"))->ToElement();
 
 	AddTextElement(pCert, "Data", ConvertHexToString(data, len));
 
-
-	wxLongLong time = pNotification->GetActivationTime().GetTicks();
+	wxLongLong time = certificate.GetActivationTime().GetTicks();
 	AddTextElement(pCert, "ActivationTime", time.ToString());
 
-	time = pNotification->GetExpirationTime().GetTicks();
+	time = certificate.GetExpirationTime().GetTicks();
 	AddTextElement(pCert, "ExpirationTime", time.ToString());
 
 	m_xmlFile.Save();
 	m_xmlFile.Close();
 }
+
+wxString CVerifyCertDialog::DecodeValue(const wxString& value)
+{
+	// Decodes string in hex notation
+	// #xxxx466F6F626172 -> Foobar
+	// First two encoded bytes are ignored, some weird type information I don't care about
+	// Only accepts ASCII for now.
+	if (value[0] != '#')
+		return value;
+
+	unsigned int len = value.Len();
+	//if (!(len % 2))
+	//	return value;
+
+	wxString out;
+
+	for (unsigned int i = 5; i < len; i += 2)
+	{
+		wxChar c = value[i];
+		wxChar d = value[i + 1];
+		if (c >= '0' && c <= '9')
+			c -= '0';
+		else if (c >= 'a' && c <= 'z')
+			c -= 'a' - 10;
+		else if (c >= 'A' && c <= 'Z')
+			c -= 'A' - 10;
+		else
+			continue;
+			//return value;
+		if (d >= '0' && d <= '9')
+			d -= '0';
+		else if (d >= 'a' && d <= 'z')
+			d -= 'a' - 10;
+		else if (d >= 'A' && d <= 'Z')
+			d -= 'A' - 10;
+		else
+			continue;
+			//return value;
+
+		c = c * 16 + d;
+		//if (c > 127)
+		//	return value;
+		if (c > 127 || c < 32)
+			continue;
+		out += c;
+	}
+
+	return out;
+}
+
+void CVerifyCertDialog::OnCertificateChoice(wxCommandEvent& event)
+{
+	int sel = event.GetSelection();
+	if (sel < 0 || sel > (int)m_certificates.size())
+		return;
+	DisplayCert(m_pDlg, m_certificates[sel]);
+
+	m_pDlg->Layout();
+	m_pDlg->GetSizer()->Fit(m_pDlg);
+}
