@@ -596,10 +596,13 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 			const int offset = 0;
 #endif
 			int res = WrapRecursive(*iter, pSizer, desiredWidth - offset);
-			didwrap.push_back(res);
 			if (res & wrap_didwrap)
 				pSizer->Layout();
-			size.IncTo(pSizer->GetMinSize());
+			didwrap.push_back(res);
+			wxSize minSize = pSizer->GetMinSize();
+			if (minSize.x > desiredWidth)
+				res |= wrap_failed;
+			size.IncTo(minSize);
 			if (res & wrap_failed)
 				break;
 		}
@@ -874,7 +877,7 @@ void CWrapEngine::SetWidthToCache(const char* name, int width)
 	TiXmlElement* pDialog = FindElementWithAttribute(pLanguage, "Dialog", "name", name);
 	if (!pDialog)
 	{
-		pDialog = pLanguage->InsertEndChild(TiXmlElement("Dialog"))->ToElement();
+		pDialog = pLanguage->LinkEndChild(new TiXmlElement("Dialog"))->ToElement();
 		pDialog->SetAttribute("name", name);
 	}
 
@@ -946,7 +949,7 @@ bool CWrapEngine::LoadCache()
 
 	TiXmlElement* pElement = pDocument->FirstChildElement("Layout");
 	if (!pElement)
-		pElement = pDocument->InsertEndChild(TiXmlElement("Layout"))->ToElement();
+		pElement = pDocument->LinkEndChild(new TiXmlElement("Layout"))->ToElement();
 
 	const wxString buildDate = CBuildInfo::GetBuildDateString();
 	if (GetTextAttribute(pElement, "Builddate") != buildDate)
@@ -967,7 +970,7 @@ bool CWrapEngine::LoadCache()
 
 	TiXmlElement* pResources = pElement->FirstChildElement("Resources");
 	if (!pResources)
-		pResources = pElement->InsertEndChild(TiXmlElement("Resources"))->ToElement();
+		pResources = pElement->LinkEndChild(new TiXmlElement("Resources"))->ToElement();
 
 	wxString resourceDir = wxGetApp().GetResourceDir();
 	wxDir dir(resourceDir);
@@ -987,7 +990,7 @@ bool CWrapEngine::LoadCache()
 		TiXmlElement* resourceElement = FindElementWithAttribute(pResources, "xrc", "file", xrc.mb_str());
 		if (!resourceElement)
 		{
-			resourceElement = pResources->InsertEndChild(TiXmlElement("xrc"))->ToElement();
+			resourceElement = pResources->LinkEndChild(new TiXmlElement("xrc"))->ToElement();
 			resourceElement->SetAttribute("file", xrc.mb_str());
 			resourceElement->SetAttribute("date", ticks.ToString().mb_str());
 			cacheValid = false;
@@ -1023,7 +1026,7 @@ bool CWrapEngine::LoadCache()
 	TiXmlElement* languageElement = FindElementWithAttribute(pElement, "Language", "id", language.mb_str());
 	if (!languageElement)
 	{
-		languageElement = pElement->InsertEndChild(TiXmlElement("Language"))->ToElement();
+		languageElement = pElement->LinkEndChild(new TiXmlElement("Language"))->ToElement();
 		languageElement->SetAttribute("id", language.mb_str());
 	}
 
@@ -1037,7 +1040,7 @@ bool CWrapEngine::LoadCache()
 
 	TiXmlElement* pFontElement = languageElement->FirstChildElement("Font");
 	if (!pFontElement)
-		pFontElement = languageElement->InsertEndChild(TiXmlElement("Font"))->ToElement();
+		pFontElement = languageElement->LinkEndChild(new TiXmlElement("Font"))->ToElement();
 
 	if (GetTextAttribute(pFontElement, "font") != fontDesc)
 	{
