@@ -3452,3 +3452,11 @@ int psftp_main(int argc, char *argv[])
 
     return 0;
 }
+
+void notify_remote_exit(void *frontend)
+{
+    int exitcode = back->exitcode(backhandle);
+
+    if (exitcode > 0)
+	connection_fatal(frontend, "Connection closed by server with exitcode %d", exitcode);
+}
